@@ -1,5 +1,3 @@
-;; invoke.lisp v2.0
-;;
 ;; Copyright (C) 2005 Alan Ruttenberg
 ;; Copyright (C) 2011-2 Mark Evenson
 ;;
@@ -147,11 +145,11 @@ NAME can either string or a symbol according to the usual JSS conventions."
 (defun maybe-resolve-class-against-imports (classname)
   (or (gethash classname *imports-resolved-classes*)
       (let ((found (lookup-class-name classname)))
-	(if found
-	    (progn 
-	      (setf (gethash classname *imports-resolved-classes*) found)
-	      found)
-	    (string classname)))))
+        (if found
+            (progn 
+              (setf (gethash classname *imports-resolved-classes*) found)
+              found)
+            (string classname)))))
 
 (defvar *class-name-to-full-case-insensitive* (make-hash-table :test 'equalp))
 
@@ -163,7 +161,7 @@ NAME can either string or a symbol according to the usual JSS conventions."
 ;; otherwise a regular method is called.
 
 (defun invoke (method object &rest args)
-    (invoke-restargs method object args))
+  (invoke-restargs method object args))
 
 (defun invoke-restargs (method object args &optional (raw? nil))
   (let* ((object-as-class-name 
@@ -227,84 +225,84 @@ want to avoid the overhead of the dynamic dispatch."
   (if (null fname-jname-pairs)
       `(progn ,@body)
       (destructuring-bind ((fname jname &optional raw) &rest ignore) fname-jname-pairs
-	(declare (ignore ignore))
-	(let ((varname (gensym)))
-	  `(let ((,varname nil))
-	     (macrolet ((,fname (&rest args)
-			  `(if ,',varname
-			       (if ,',raw
-				   (jcall-raw ,',varname ,@args)
-				   (jcall ,',varname ,@args))
-			       (progn
-				 (setq ,',varname (invoke-find-method ,',jname ,(car args) (list ,@(rest args))))
-				 (if ,',raw
-				     (jcall-raw ,',varname ,@args)
-				     (jcall ,',varname ,@args))))))
-	       (with-constant-signature ,(cdr fname-jname-pairs)
-		 ,@body)))))))
+        (declare (ignore ignore))
+        (let ((varname (gensym)))
+          `(let ((,varname nil))
+             (macrolet ((,fname (&rest args)
+                          `(if ,',varname
+                               (if ,',raw
+                                   (jcall-raw ,',varname ,@args)
+                                   (jcall ,',varname ,@args))
+                               (progn
+                                 (setq ,',varname (invoke-find-method ,',jname ,(car args) (list ,@(rest args))))
+                                 (if ,',raw
+                                     (jcall-raw ,',varname ,@args)
+                                     (jcall ,',varname ,@args))))))
+               (with-constant-signature ,(cdr fname-jname-pairs)
+                 ,@body)))))))
 
 (defun lookup-class-name (name)
   (setq name (string name))
   (let* (;; cant (last-name-pattern (#"compile" '|java.util.regex.Pattern| ".*?([^.]*)$"))
-	 ;; reason: bootstrap - the class name would have to be looked up...
-	 (last-name-pattern (load-time-value (jstatic (jmethod "java.util.regex.Pattern" "compile"
-							       (jclass "java.lang.String"))
-						      (jclass "java.util.regex.Pattern") 
-						      ".*?([^.]*)$")))
-	 (last-name 
-	  (let ((matcher (#0"matcher" last-name-pattern name)))
-	    (#"matches" matcher)
-	    (#"group" matcher 1))))
+         ;; reason: bootstrap - the class name would have to be looked up...
+         (last-name-pattern (load-time-value (jstatic (jmethod "java.util.regex.Pattern" "compile"
+                                                               (jclass "java.lang.String"))
+                                                      (jclass "java.util.regex.Pattern") 
+                                                      ".*?([^.]*)$")))
+         (last-name 
+          (let ((matcher (#0"matcher" last-name-pattern name)))
+            (#"matches" matcher)
+            (#"group" matcher 1))))
     (let* ((bucket (gethash last-name *class-name-to-full-case-insensitive*))
-	   (bucket-length (length bucket)))
+           (bucket-length (length bucket)))
       (or (find name bucket :test 'equalp)
-	  (flet ((matches-end (end full test)
-		   (= (+ (or (search end full :from-end t :test test) -10)
-			 (length end))
-		      (length full)))
-		 (ambiguous (choices)
-		   (error "Ambiguous class name: ~a can be ~{~a~^, ~}" name choices)))
-	    (if (zerop bucket-length)
-		name
-		(let ((matches (loop for el in bucket when (matches-end name el 'char=) collect el)))
-		  (if (= (length matches) 1)
-		      (car matches)
-		      (if (= (length matches) 0)
-			  (let ((matches (loop for el in bucket when (matches-end name el 'char-equal) collect el)))
-			    (if (= (length matches) 1)
-				(car matches)
-				(if (= (length matches) 0)
-				    name
-				    (ambiguous matches))))
-			  (ambiguous matches))))))))))
+          (flet ((matches-end (end full test)
+                   (= (+ (or (search end full :from-end t :test test) -10)
+                         (length end))
+                      (length full)))
+                 (ambiguous (choices)
+                   (error "Ambiguous class name: ~a can be ~{~a~^, ~}" name choices)))
+            (if (zerop bucket-length)
+                name
+                (let ((matches (loop for el in bucket when (matches-end name el 'char=) collect el)))
+                  (if (= (length matches) 1)
+                      (car matches)
+                      (if (= (length matches) 0)
+                          (let ((matches (loop for el in bucket when (matches-end name el 'char-equal) collect el)))
+                            (if (= (length matches) 1)
+                                (car matches)
+                                (if (= (length matches) 0)
+                                    name
+                                    (ambiguous matches))))
+                          (ambiguous matches))))))))))
 
 (defun get-all-jar-classnames (jar-file-name)
   (let* ((jar (jnew (jconstructor "java.util.jar.JarFile" (jclass "java.lang.String")) (namestring (truename jar-file-name))))
          (entries (#"entries" jar)))
     (with-constant-signature ((matcher "matcher" t) (substring "substring")
-			      (jreplace "replace" t) (jlength "length")
-			      (matches "matches") (getname "getName" t)
-			      (next "nextElement" t) (hasmore "hasMoreElements")
-			      (group "group"))
+                              (jreplace "replace" t) (jlength "length")
+                              (matches "matches") (getname "getName" t)
+                              (next "nextElement" t) (hasmore "hasMoreElements")
+                              (group "group"))
       (loop while (hasmore entries)
-	 for name =  (getname (next entries))
-	 with class-pattern = (#"compile" '|java.util.regex.Pattern| "[^$]*\\.class$")
-	 with name-pattern = (#"compile" '|java.util.regex.Pattern| ".*?([^.]*)$")
-	 when (matches (matcher class-pattern name))
-	 collect
-	   (let* ((fullname (substring (jreplace name #\/ #\.) 0 (- (jlength name) 6)))
-		  (matcher (matcher name-pattern fullname))
-		  (name (progn (matches matcher) (group matcher 1))))
-	     (cons name fullname))
-	 ))))
+         for name =  (getname (next entries))
+         with class-pattern = (#"compile" '|java.util.regex.Pattern| "[^$]*\\.class$")
+         with name-pattern = (#"compile" '|java.util.regex.Pattern| ".*?([^.]*)$")
+         when (matches (matcher class-pattern name))
+         collect
+           (let* ((fullname (substring (jreplace name #\/ #\.) 0 (- (jlength name) 6)))
+                  (matcher (matcher name-pattern fullname))
+                  (name (progn (matches matcher) (group matcher 1))))
+             (cons name fullname))
+           ))))
 
 (defun jar-import (file)
   "Import all the Java classes contained in the pathname FILE into the JSS dynamic lookup cache."
   (when (probe-file file)
     (loop for (name . full-class-name) in (get-all-jar-classnames file)
        do 
-	 (pushnew full-class-name (gethash name *class-name-to-full-case-insensitive*) 
-		  :test 'equal))))
+         (pushnew full-class-name (gethash name *class-name-to-full-case-insensitive*) 
+                  :test 'equal))))
 
 (defun new (class-name &rest args)
   "Invoke the Java constructor for CLASS-NAME with ARGS.
@@ -314,26 +312,57 @@ CLASS-NAME may either be a symbol or a string according to the usual JSS convent
 
 (defvar *running-in-osgi* (ignore-errors (jclass "org.osgi.framework.BundleActivator")))
 
+(define-condition no-such-java-field (error)
+  ((field-name
+    :initarg :field-name
+    :reader field-name
+    )
+   (object
+    :initarg :object
+    :reader object
+    ))
+  (:report (lambda (c stream)
+             (format stream "Unable to find a FIELD named ~a for ~a"
+                     (field-name c) (object c))))
+  )
+
 (defun get-java-field (object field &optional (try-harder *running-in-osgi*))
   "Get the value of the FIELD contained in OBJECT.
 If OBJECT is a symbol it names a dot qualified static FIELD."
   (if try-harder
       (let* ((class (if (symbolp object)
-			(setq object (find-java-class object))
+                        (setq object (find-java-class object))
                         (if (equal "java.lang.Class" (jclass-name (jobject-class object)))
                             object
                             (jobject-class object))))
-	     (jfield (if (java-object-p field)
-			 field
-                         (find field (#"getDeclaredFields" class) 
-                               :key 'jfield-name :test 'equal))))
-	(#"setAccessible" jfield +true+)
-	(values (#"get" jfield object) jfield))
+             (jfield (if (java-object-p field)
+                         field
+                         (or (find-declared-field field class)
+                             (error 'no-such-java-field :field-name field :object object)))))
+        (#"setAccessible" jfield +true+)
+        (values (#"get" jfield object) jfield))
       (if (symbolp object)
           (let ((class (find-java-class object)))
             (jfield class field))
           (jfield field object))))
 
+(defun find-declared-field (field class)
+  "Return a FIELD object corresponding to the definition of FIELD
+\(a string\) visible at CLASS. *Not* restricted to public classes, and checks
+all superclasses of CLASS.
+   Returns NIL if no field object is found."
+  (loop while class
+     for field-obj = (get-declared-field class field)
+     if field-obj
+     do (return-from find-declared-field field-obj)
+     else
+     do (setf class (jclass-superclass class)))
+  nil)
+
+(defun get-declared-field (class fieldname)
+  (find fieldname (#"getDeclaredFields" class)
+        :key 'jfield-name :test 'equal))
+
 ;; TODO use #"getSuperclass" and #"getInterfaces" to see whether there
 ;; are fields in superclasses that we might set
 (defun set-java-field (object field value &optional (try-harder *running-in-osgi*))
@@ -343,21 +372,25 @@ a static FIELD.  If OBJECT is an instance of java:java-object, the
 associated is used to look up the static FIELD."
   (if try-harder
       (let* ((class (if (symbolp object)
-			(setq object (find-java-class object))
-		      (if (equal "java.lang.Class" (jclass-name (jobject-class object)) )
-			  object
-			(jobject-class object))))
-	     (jfield (if (java-object-p field)
-			 field
-		       (find field (#"getDeclaredFields" class) :key 'jfield-name :test 'equal))))
-	(#"setAccessible" jfield +true+)
-	(values (#"set" jfield object value) jfield))
-    (if (symbolp object)
-	(let ((class (find-java-class object)))
-          (setf (jfield (#"getName" class) field) value))
-        (if (typep object 'java-object)
-            (setf (jfield (jclass-of object) field) value)
-            (setf (jfield object field) value)))))
+                        (setq object (find-java-class object))
+                        (if (equal "java.lang.Class" (jclass-name (jobject-class object)) )
+                            object
+                            (jobject-class object))))
+             (jfield (if (java-object-p field)
+                         field
+                         (or (find-declared-field field class)
+                             (error 'no-such-java-field :field-name field :object object)))))
+        (#"setAccessible" jfield +true+)
+        (values (#"set" jfield object value) jfield))
+      (if (symbolp object)
+          (let ((class (find-java-class object)))
+            (setf (jfield (#"getName" class) field) value))
+          (if (typep object 'java-object)
+              (setf (jfield (jclass-of object) field) value)
+              (setf (jfield object field) value)))))
+
+(defun (setf get-java-field) (value object field &optional (try-harder *running-in-osgi*))
+  (set-java-field object field value try-harder))
 
 
 (defconstant +for-name+ 
@@ -378,31 +411,31 @@ associated is used to look up the static FIELD."
 
 (defun do-auto-imports ()
   (labels ((expand-paths (cp)
-           (loop :for s :in cp
-              :appending (loop :for entry 
-                            :in (let ((p (pathname s)))
-                                  (if (wild-pathname-p p)
-                                      (directory p)
-                                      (list p)))
-                            :collecting entry)))
-         (import-classpath (cp)
-	   (mapcar 
-		(lambda (p) 
-		  (when *load-verbose*
-		    (format t ";; Importing ~A~%" p))
-		  (cond 
-		    ((file-directory-p p) )
-		    ((equal (pathname-type p) "jar")
-		     (jar-import (merge-pathnames p
-                                                  (format nil "~a/" (jstatic "getProperty" "java.lang.System" "user.dir")))))))
-                cp))
-         (split-classpath (cp)
-           (coerce 
-            (jcall "split" cp 
-                   (string (jfield (jclass "java.io.File") "pathSeparatorChar")))
-            'cons))
-         (do-imports (cp)
-           (import-classpath (expand-paths (split-classpath cp)))))
+             (loop :for s :in cp
+                :appending (loop :for entry 
+                              :in (let ((p (pathname s)))
+                                    (if (wild-pathname-p p)
+                                        (directory p)
+                                        (list p)))
+                              :collecting entry)))
+           (import-classpath (cp)
+             (mapcar 
+              (lambda (p) 
+                (when *load-verbose*
+                  (format t ";; Importing ~A~%" p))
+                (cond 
+                  ((file-directory-p p) )
+                  ((equal (pathname-type p) "jar")
+                   (jar-import (merge-pathnames p
+                                                (format nil "~a/" (jstatic "getProperty" "java.lang.System" "user.dir")))))))
+              cp))
+           (split-classpath (cp)
+             (coerce 
+              (jcall "split" cp 
+                     (string (jfield (jclass "java.io.File") "pathSeparatorChar")))
+              'cons))
+           (do-imports (cp)
+             (import-classpath (expand-paths (split-classpath cp)))))
     (do-imports (jcall "getClassPath" (jstatic "getRuntimeMXBean" '|java.lang.management.ManagementFactory|)))
     (do-imports (jcall "getBootClassPath" (jstatic "getRuntimeMXBean" '|java.lang.management.ManagementFactory|)))))
 
@@ -411,24 +444,24 @@ associated is used to look up the static FIELD."
     (do-auto-imports)))
 
 (defun japropos (string)
-"Output the names of all Java class names loaded in the current process which match STRING.."
+  "Output the names of all Java class names loaded in the current process which match STRING.."
   (setq string (string string))
   (let ((matches nil))
     (maphash (lambda(key value) 
-	       (declare (ignore key))
-	       (loop for class in value
-		  when (search string class :test 'string-equal)
-		    do (pushnew (list class "Java Class") matches :test 'equal)))
-	     *class-name-to-full-case-insensitive*)
+               (declare (ignore key))
+               (loop for class in value
+                  when (search string class :test 'string-equal)
+                  do (pushnew (list class "Java Class") matches :test 'equal)))
+             *class-name-to-full-case-insensitive*)
     (loop for (match type) in (sort matches 'string-lessp :key 'car)
-	 do (format t "~a: ~a~%" match type))
+       do (format t "~a: ~a~%" match type))
     ))
 
 (defun jclass-method-names (class &optional full)
   (if (java-object-p class)
       (if (equal (jclass-name (jobject-class class)) "java.lang.Class")
-	  (setq class (jclass-name class))
-	  (setq class (jclass-name (jobject-class class)))))
+          (setq class (jclass-name class))
+          (setq class (jclass-name (jobject-class class)))))
   (union
    (remove-duplicates (map 'list (if full #"toString" 'jmethod-name) (#"getMethods" (find-java-class class))) :test 'equal)
    (ignore-errors (remove-duplicates (map 'list (if full #"toString" 'jmethod-name) (#"getConstructors" (find-java-class class))) :test 'equal))))
@@ -443,7 +476,7 @@ notation, or a symbol resolved against all class entries in the
 current classpath."
   (if stream
       (dolist (method (jclass-method-names class t))
-	(format stream "~a~%" method))
+        (format stream "~a~%" method))
       (jclass-method-names class)))
 
 (setf (symbol-function 'jcmn) 'java-class-method-names)
@@ -459,19 +492,19 @@ current classpath."
 
 (defun all-loaded-classes ()
   (let ((classes-field 
-	 (find "classes" (#"getDeclaredFields" (jclass "java.lang.ClassLoader"))
-	       :key #"getName" :test 'equal)))
+         (find "classes" (#"getDeclaredFields" (jclass "java.lang.ClassLoader"))
+               :key #"getName" :test 'equal)))
     (#"setAccessible" classes-field +true+)
     (loop for classloader in (mapcar #'first (dump-classpath))
-	 append
-	 (loop with classesv = (#"get" classes-field classloader)
-	    for i below (#"size" classesv)
-	    collect (#"getName" (#"elementAt" classesv i)))
-	 append
-	 (loop with classesv = (#"get" classes-field (#"getParent" classloader))
-	    for i below (#"size" classesv)
-	    collect (#"getName" (#"elementAt" classesv i))))))
-	 
+       append
+         (loop with classesv = (#"get" classes-field classloader)
+            for i below (#"size" classesv)
+            collect (#"getName" (#"elementAt" classesv i)))
+       append
+         (loop with classesv = (#"get" classes-field (#"getParent" classloader))
+            for i below (#"size" classesv)
+            collect (#"getName" (#"elementAt" classesv i))))))
+
 (defun get-dynamic-class-path ()
   (rest 
    (find-if (lambda (loader) 
@@ -489,9 +522,9 @@ current classpath."
 (defun java-room ()
   (let ((rt (#"getRuntime" 'java.lang.runtime)))
     (values (- (#"totalMemory" rt) (#"freeMemory" rt))
-	   (#"totalMemory" rt)
-	   (#"freeMemory" rt)
-	   (list :used :total :free))))
+            (#"totalMemory" rt)
+            (#"freeMemory" rt)
+            (list :used :total :free))))
 
 (defun verbose-gc (&optional (new-value nil new-value-supplied))
   (if new-value-supplied
@@ -510,23 +543,23 @@ current classpath."
      if (null (pathname-name top)) do (setq q (append q (all-classfiles-below top ))) 
      if (equal (pathname-type top) "class")
      collect top
-     ))
+       ))
 
 (defun all-classes-below-directory (directory)
   (loop for file in (all-classfiles-below directory) collect
        (format nil "~{~a.~}~a"
-	       (subseq (pathname-directory file) (length (pathname-directory directory)))
-	       (pathname-name file))
+               (subseq (pathname-directory file) (length (pathname-directory directory)))
+               (pathname-name file))
        ))
 
 (defun classfiles-import (directory)
   "Load all Java classes recursively contained under DIRECTORY in the current process."
   (setq directory (truename directory))
   (loop for full-class-name in (all-classes-below-directory directory)
-       for name = (#"replaceAll" full-class-name "^.*\\." "")
+     for name = (#"replaceAll" full-class-name "^.*\\." "")
      do
        (pushnew full-class-name (gethash name *class-name-to-full-case-insensitive*) 
-		:test 'equal)))
+                :test 'equal)))
 
 (defun set-to-list (set)
   (declare (optimize (speed 3) (safety 0)))
@@ -565,24 +598,24 @@ current classpath."
 
 (defun iterable-to-list (iterable)
   "Return the items contained the java.lang.Iterable ITERABLE as a list."
- (declare (optimize (speed 3) (safety 0)))
- (let ((it (#"iterator" iterable)))
-   (with-constant-signature ((has-next "hasNext")
-			     (next "next"))
-     (loop :while (has-next it)
-	:collect (next it)))))
+  (declare (optimize (speed 3) (safety 0)))
+  (let ((it (#"iterator" iterable)))
+    (with-constant-signature ((has-next "hasNext")
+                              (next "next"))
+      (loop :while (has-next it)
+         :collect (next it)))))
 
 (defun vector-to-list (vector)
   "Return the elements of java.lang.Vector VECTOR as a list."
- (declare (optimize (speed 3) (safety 0)))
- (with-constant-signature ((has-more "hasMoreElements")
-			   (next "nextElement"))
-   (let ((elements (#"elements" vector)))
-     (loop :while (has-more elements)
-	:collect (next elements)))))
+  (declare (optimize (speed 3) (safety 0)))
+  (with-constant-signature ((has-more "hasMoreElements")
+                            (next "nextElement"))
+    (let ((elements (#"elements" vector)))
+      (loop :while (has-more elements)
+         :collect (next elements)))))
 
 (defun hashmap-to-hashtable (hashmap &rest rest &key (keyfun #'identity) (valfun #'identity) (invert? nil)
-				    table 
+                                                  table 
                              &allow-other-keys )
   "Converts the a HASHMAP reference to a java.util.HashMap object to a Lisp hashtable.
 
@@ -593,19 +626,19 @@ of the HASHMAP right before they are placed in the hashtable.
 
 If INVERT? is non-nil than reverse the keys and values in the resulting hashtable."
   (let ((keyset (#"keySet" hashmap))
-	(table (or table (apply 'make-hash-table
-				(loop for (key value) on rest by #'cddr
-				   unless (member key '(:invert? :valfun :keyfun :table)) 
-				   collect key and collect value)))))
+        (table (or table (apply 'make-hash-table
+                                (loop for (key value) on rest by #'cddr
+                                   unless (member key '(:invert? :valfun :keyfun :table)) 
+                                   collect key and collect value)))))
     (with-constant-signature ((iterator "iterator" t) (hasnext "hasNext") (next "next"))
       (loop with iterator = (iterator keyset)
-	 while (hasNext iterator)
-	 for item = (next iterator)
-	 do (if invert?
-		(setf (gethash (funcall valfun (#"get" hashmap item)) table) (funcall keyfun item))
-		(setf (gethash (funcall keyfun item) table) (funcall valfun (#"get" hashmap item)))))
-    table)))
-	   
+         while (hasNext iterator)
+         for item = (next iterator)
+         do (if invert?
+                (setf (gethash (funcall valfun (#"get" hashmap item)) table) (funcall keyfun item))
+                (setf (gethash (funcall keyfun item) table) (funcall valfun (#"get" hashmap item)))))
+      table)))
+
 (defun jclass-all-interfaces (class)
   "Return a list of interfaces the class implements"
   (unless (java-object-p class)
@@ -617,10 +650,10 @@ If INVERT? is non-nil than reverse the keys and values in the resulting hashtabl
 (defun safely (f name)
   (let ((fname (gensym)))
     (compile fname
-	     `(lambda(&rest args)
-		(with-simple-restart (top-level
-				      "Return from lisp method implementation for ~a." ,name)
-		  (apply ,f args))))
+             `(lambda(&rest args)
+                (with-simple-restart (top-level
+                                      "Return from lisp method implementation for ~a." ,name)
+                  (apply ,f args))))
     (symbol-function fname)))
 
 (defun jdelegating-interface-implementation (interface dispatch-to &rest method-names-and-defs)
@@ -638,25 +671,25 @@ If INVERT? is non-nil than reverse the keys and values in the resulting hashtabl
    calls the method on DISPATCH-TO."
   (let ((implemented-methods
          (loop for m in method-names-and-defs
-	    for i from 0
-	    if (evenp i) 
-	    do (assert (stringp m) (m) "Method names must be strings: ~s" m) and collect m
-	    else
-	    do (assert (or (symbolp m) (functionp m)) (m) "Methods must be function designators: ~s" m))))
+            for i from 0
+            if (evenp i) 
+            do (assert (stringp m) (m) "Method names must be strings: ~s" m) and collect m
+            else
+            do (assert (or (symbolp m) (functionp m)) (m) "Methods must be function designators: ~s" m))))
     (let ((safe-method-names-and-defs 
-	   (loop for (name function) on method-names-and-defs by #'cddr
-	      collect name collect (safely function name))))
+           (loop for (name function) on method-names-and-defs by #'cddr
+              collect name collect (safely function name))))
       (loop for method across
-	   (jclass-methods interface :declared nil :public t)
-	   for method-name = (jmethod-name method)
-	   when (not (member method-name implemented-methods :test #'string=))
-	   do
-	   (let* ((def  `(lambda
-			     (&rest args)
-			   (invoke-restargs ,(jmethod-name method) ,dispatch-to args t)
-			   )))
-	     (push (coerce def 'function) safe-method-names-and-defs)
-	     (push method-name safe-method-names-and-defs)))
+           (jclass-methods interface :declared nil :public t)
+         for method-name = (jmethod-name method)
+         when (not (member method-name implemented-methods :test #'string=))
+         do
+           (let* ((def  `(lambda
+                             (&rest args)
+                           (invoke-restargs ,(jmethod-name method) ,dispatch-to args t)
+                           )))
+             (push (coerce def 'function) safe-method-names-and-defs)
+             (push method-name safe-method-names-and-defs)))
       (apply #'java::%jnew-proxy  interface safe-method-names-and-defs))))
 
 
