@@ -2,7 +2,7 @@
  * DoubleFloat.java
  *
  * Copyright (C) 2003-2007 Peter Graves
- * $Id$
+ * $Id: DoubleFloat.java 14757 2015-04-11 07:44:42Z mevenson $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -169,16 +169,8 @@ public final class DoubleFloat extends LispObject
     @Override
     public boolean equalp(LispObject obj)
     {
-        if (obj instanceof SingleFloat)
-            return value == ((SingleFloat)obj).value;
-        if (obj instanceof DoubleFloat)
-            return value == ((DoubleFloat)obj).value;
-        if (obj instanceof Fixnum)
-            return value == ((Fixnum)obj).value;
-        if (obj instanceof Bignum)
-            return value == ((Bignum)obj).doubleValue();
-        if (obj instanceof Ratio)
-            return value == ((Ratio)obj).doubleValue();
+        if (obj != null && obj.numberp())
+            return isEqualTo(obj);
         return false;
     }
 
