@@ -2,7 +2,7 @@
  * Fixnum.java
  *
  * Copyright (C) 2002-2006 Peter Graves
- * $Id$
+ * $Id: Fixnum.java 14757 2015-04-11 07:44:42Z mevenson $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -228,12 +228,8 @@ public final class Fixnum extends LispInteger
   @Override
   public boolean equalp(LispObject obj)
   {
-    if (obj instanceof Fixnum)
-      return value == ((Fixnum)obj).value;
-    if (obj instanceof SingleFloat)
-      return value == ((SingleFloat)obj).value;
-    if (obj instanceof DoubleFloat)
-      return value == ((DoubleFloat)obj).value;
+    if (obj != null && obj.numberp())
+      return isEqualTo(obj);
     return false;
   }
 
