@@ -2,7 +2,7 @@
  * FloatFunctions.java
  *
  * Copyright (C) 2003-2006 Peter Graves
- * $Id: FloatFunctions.java 13440 2011-08-05 21:25:10Z ehuelsmann $
+ * $Id: FloatFunctions.java 14465 2013-04-24 12:50:37Z rschlatte $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -48,7 +48,7 @@ public final class FloatFunctions
         public LispObject execute(LispObject[] args)
         {
             if (args.length % 2 != 0)
-                error(new ProgramError("Odd number of keyword arguments."));
+                program_error("Odd number of keyword arguments.");
             for (int i = 0; i < args.length; i += 2) {
                 LispObject key = checkSymbol(args[i]);
                 LispObject value = args[i+1];
