@@ -2,7 +2,7 @@
  * Interpreter.java
  *
  * Copyright (C) 2002-2006 Peter Graves
- * $Id: Interpreter.java 14089 2012-08-14 21:06:34Z ehuelsmann $
+ * $Id: Interpreter.java 14591 2013-12-27 15:58:16Z mevenson $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -595,10 +595,19 @@ public final class Interpreter
             initializeJLisp();
         StringInputStream stream = new StringInputStream(s);
         final LispThread thread = LispThread.currentThread();
-        LispObject obj = stream.read(false, EOF, false, thread,
-                                     Stream.currentReadtable);
+        LispObject obj = null;
+
+        final SpecialBindingsMark mark0 = thread.markSpecialBindings();
+        thread.bindSpecial(Symbol.DEBUGGER_HOOK, _DEBUGGER_HOOK_FUNCTION);
+        try {  // catch possible errors from use of SHARPSIGN_DOT macros in --eval stanzas
+          obj = stream.read(false, EOF, false, thread,
+                            Stream.currentReadtable);
+        } finally {
+          thread.resetSpecialBindings(mark0);
+        }
         if (obj == EOF)
             return error(new EndOfFile(stream));
+
         final SpecialBindingsMark mark = thread.markSpecialBindings();
         thread.bindSpecial(Symbol.DEBUGGER_HOOK, _DEBUGGER_HOOK_FUNCTION);
         try {
