@@ -2,7 +2,7 @@
  * LispThread.java
  *
  * Copyright (C) 2003-2007 Peter Graves
- * $Id: LispThread.java 14025 2012-07-29 20:11:29Z ehuelsmann $
+ * $Id: LispThread.java 14690 2014-04-22 11:24:50Z mevenson $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -214,7 +214,7 @@ public final class LispThread extends LispObject
         return values;
     }
 
-    // Used by the JVM compiler for MULTIPLE-VALUE-CALL.
+    /** Used by the JVM compiler for MULTIPLE-VALUE-CALL. */
     public final LispObject[] accumulateValues(LispObject result,
                                                LispObject[] oldValues)
     {
@@ -315,8 +315,10 @@ public final class LispThread extends LispObject
         return NIL;
     }
 
-    // Forces a single value, for situations where multiple values should be
-    // ignored.
+   /** 
+    * Force a single value, for situations where multiple values should be
+    * ignored.
+    */
     public final LispObject value(LispObject obj)
     {
         _values = null;
@@ -597,50 +599,222 @@ public final class LispThread extends LispObject
     }
 
 
-    private StackFrame stack = null;
+    private static class StackMarker {
 
-   public final void pushStackFrame(StackFrame frame)
-    {
-        frame.setNext(stack);
-        stack = frame;
+        final int numArgs;
+
+        StackMarker(int numArgs) {
+            this.numArgs = numArgs;
+        }
+
+        int getNumArgs() {
+            return numArgs;
+        }
     }
 
-    public final void popStackFrame()
+    // markers for args
+    private final static StackMarker STACK_MARKER_0 = new StackMarker(0);
+    private final static StackMarker STACK_MARKER_1 = new StackMarker(1);
+    private final static StackMarker STACK_MARKER_2 = new StackMarker(2);
+    private final static StackMarker STACK_MARKER_3 = new StackMarker(3);
+    private final static StackMarker STACK_MARKER_4 = new StackMarker(4);
+    private final static StackMarker STACK_MARKER_5 = new StackMarker(5);
+    private final static StackMarker STACK_MARKER_6 = new StackMarker(6);
+    private final static StackMarker STACK_MARKER_7 = new StackMarker(7);
+    private final static StackMarker STACK_MARKER_8 = new StackMarker(8);
+
+    private final int STACK_FRAME_EXTRA = 2;
+    // a LispStackFrame with n arguments occupies n + STACK_FRAME_EXTRA elements
+    // in {@code stack} array.
+    // stack[framePos] == operation
+    // stack[framePos + 1 + i] == arg[i]
+    // stack[framePos + 1 + n] == initially SrackMarker(n)
+    // LispStackFrame object may be lazily allocated later.
+    // In this case it is stored in stack framePos + 1 + n]
+    //
+    // Java stack frame occupies 1 element
+    // stack[framePos] == JavaStackFrame
+    //
+    // Stack consists of a list of StackSegments.
+    // Top StackSegment is cached in variables stack and stackPtr.
+    private StackSegment topStackSegment = new StackSegment(INITIAL_SEGMENT_SIZE, null);
+    private Object[] stack = topStackSegment.stack;
+    private int stackPtr = 0;
+    private StackSegment spareStackSegment;
+    
+    private static class StackSegment 
+      implements org.armedbear.lisp.protocol.Inspectable
     {
-        if (stack != null)
-            stack = stack.getNext();
+        final Object[] stack;
+        final StackSegment next;
+        int stackPtr;
+        
+        StackSegment(int size, StackSegment next) {
+            stack = new Object[size];
+            this.next = next;
+        }
+        public LispObject getParts() {
+        Cons result = new Cons(NIL);
+        return result
+          .push(new Symbol("INITIAL-SEGMENT-SIZE"))
+            .push(LispInteger.getInstance(LispThread.INITIAL_SEGMENT_SIZE))
+          .push(new Symbol("SEGMENT-SIZE"))
+            .push(LispInteger.getInstance(LispThread.SEGMENT_SIZE)).nreverse();
+        }
+    }
+    
+    private void ensureStackCapacity(int itemsToPush) {
+        if (stackPtr + (itemsToPush - 1) >= stack.length)
+            grow(itemsToPush);
+    }
+
+    private static final int INITIAL_SEGMENT_SIZE = 1 << 10;
+    private static final int SEGMENT_SIZE = (1 << 19) - 4; // 4 MiB page on x86_64
+
+    private void grow(int numEntries) {
+        topStackSegment.stackPtr = stackPtr;
+        if (spareStackSegment != null) {
+            // Use spare segement if available
+            if (stackPtr > 0 && spareStackSegment.stack.length >= numEntries) {
+                topStackSegment = spareStackSegment;
+                stack = topStackSegment.stack;
+                spareStackSegment = null;
+                stackPtr = 0;
+                return;
+            }
+            spareStackSegment = null;
+        }
+        int newSize = stackPtr + numEntries;
+        if (topStackSegment.stack.length < SEGMENT_SIZE || stackPtr == 0) {
+            // grow initial segment from initial size to standard size
+            int newLength = Math.max(newSize, Math.min(SEGMENT_SIZE, stack.length * 2));
+            StackSegment newSegment = new StackSegment(newLength, topStackSegment.next);
+            System.arraycopy(stack, 0, newSegment.stack, 0, stackPtr);
+            topStackSegment = newSegment;
+            stack = topStackSegment.stack;
+            return;
+        }
+        // Allocate new segment
+        topStackSegment = new StackSegment(Math.max(SEGMENT_SIZE, numEntries), topStackSegment);
+        stack = topStackSegment.stack;
+        stackPtr = 0;
+    }
+
+    private StackFrame getStackTop() {
+        topStackSegment.stackPtr = stackPtr;
+        if (stackPtr == 0) {
+            assert topStackSegment.next == null;
+            return null;
+        }
+        StackFrame prev = null;
+        for (StackSegment segment = topStackSegment; segment != null; segment = segment.next) {
+            Object[] stk = segment.stack;
+            int framePos = segment.stackPtr;
+            while (framePos > 0) {
+                Object stackObj = stk[framePos - 1];
+                if (stackObj instanceof StackFrame) {
+                    if (prev != null) {
+                        prev.setNext((StackFrame) stackObj);
+                    }
+                    return (StackFrame) stack[stackPtr - 1];
+                }
+                StackMarker marker = (StackMarker) stackObj;
+                int numArgs = marker.getNumArgs();
+                LispStackFrame frame = new LispStackFrame(stk, framePos - numArgs - STACK_FRAME_EXTRA, numArgs);
+                stk[framePos - 1] = frame;
+                if (prev != null) {
+                    prev.setNext(frame);
+                }
+                prev = frame;
+                framePos -= numArgs + STACK_FRAME_EXTRA;
+            }
+        }
+        return (StackFrame) stack[stackPtr - 1];
+    }
+    
+    public final void pushStackFrame(JavaStackFrame frame) {
+        frame.setNext(getStackTop());
+        ensureStackCapacity(1);
+        stack[stackPtr] = frame;
+        stackPtr += 1;
+    }
+
+    private void popStackFrame(int numArgs) {
+        // Pop off intervening JavaFrames until we get back to a LispFrame
+        Object stackObj = stack[stackPtr - 1];
+        if (stackObj instanceof StackMarker) {
+            assert numArgs == ((StackMarker) stackObj).getNumArgs();
+        } else {
+            while (stackObj instanceof JavaStackFrame) {
+                stack[--stackPtr] = null;
+                stackObj = stack[stackPtr - 1];
+            }
+            if (stackObj instanceof StackMarker) {
+                assert numArgs == ((StackMarker) stackObj).getNumArgs();
+            } else {
+                assert numArgs == ((LispStackFrame) stackObj).getNumArgs();
+            }
+        }
+        stackPtr -= numArgs + STACK_FRAME_EXTRA;
+        for (int i = 0; i < numArgs + STACK_FRAME_EXTRA; i++) {
+            stack[stackPtr + i] = null;
+        }
+        if (stackPtr == 0) {
+            popStackSegment();
+        }
+    }
+    
+    private void popStackSegment() {
+        topStackSegment.stackPtr = 0;
+        if (topStackSegment.next != null) {
+            spareStackSegment = topStackSegment;
+            topStackSegment = topStackSegment.next;
+            stack = topStackSegment.stack;
+        }
+        stackPtr = topStackSegment.stackPtr;
     }
 
     public final Environment setEnv(Environment env) {
-        return (stack != null) ? stack.setEnv(env) : null;
+        StackFrame stackTop = getStackTop();
+        return (stackTop != null) ? stackTop.setEnv(env) : null;
     }
 
     public void resetStack()
     {
-        stack = null;
+        topStackSegment = new StackSegment(INITIAL_SEGMENT_SIZE, null);
+        stack = topStackSegment.stack;
+        spareStackSegment = null;
+        stackPtr = 0;
     }
 
     @Override
     public LispObject execute(LispObject function)
     {
-        pushStackFrame(new LispStackFrame(function));
+        ensureStackCapacity(STACK_FRAME_EXTRA);
+        stack[stackPtr] = function;
+        stack[stackPtr + 1] = STACK_MARKER_0;
+        stackPtr += STACK_FRAME_EXTRA;
         try {
             return function.execute();
         }
         finally {
-            popStackFrame();
+            popStackFrame(0);
         }
     }
 
     @Override
     public LispObject execute(LispObject function, LispObject arg)
     {
-        pushStackFrame(new LispStackFrame(function, arg));
+        ensureStackCapacity(1 + STACK_FRAME_EXTRA);
+        stack[stackPtr] = function;
+        stack[stackPtr + 1] = arg;
+        stack[stackPtr + 2] = STACK_MARKER_1;
+        stackPtr += 1 + STACK_FRAME_EXTRA;
         try {
             return function.execute(arg);
         }
         finally {
-            popStackFrame();
+            popStackFrame(1);
         }
     }
 
@@ -648,12 +822,17 @@ public final class LispThread extends LispObject
     public LispObject execute(LispObject function, LispObject first,
                               LispObject second)
     {
-        pushStackFrame(new LispStackFrame(function, first, second));
+        ensureStackCapacity(2 + STACK_FRAME_EXTRA);
+        stack[stackPtr] = function;
+        stack[stackPtr + 1] = first;
+        stack[stackPtr + 2] = second;
+        stack[stackPtr + 3] = STACK_MARKER_2;
+        stackPtr += 2 + STACK_FRAME_EXTRA;
         try {
             return function.execute(first, second);
         }
         finally {
-            popStackFrame();
+            popStackFrame(2);
         }
     }
 
@@ -661,12 +840,18 @@ public final class LispThread extends LispObject
     public LispObject execute(LispObject function, LispObject first,
                               LispObject second, LispObject third)
     {
-        pushStackFrame(new LispStackFrame(function, first, second, third));
+        ensureStackCapacity(3 + STACK_FRAME_EXTRA);
+        stack[stackPtr] = function;
+        stack[stackPtr + 1] = first;
+        stack[stackPtr + 2] = second;
+        stack[stackPtr + 3] = third;
+        stack[stackPtr + 4] = STACK_MARKER_3;
+        stackPtr += 3 + STACK_FRAME_EXTRA;
         try {
             return function.execute(first, second, third);
         }
         finally {
-            popStackFrame();
+            popStackFrame(3);
         }
     }
 
@@ -675,12 +860,19 @@ public final class LispThread extends LispObject
                               LispObject second, LispObject third,
                               LispObject fourth)
     {
-        pushStackFrame(new LispStackFrame(function, first, second, third, fourth));
+        ensureStackCapacity(4 + STACK_FRAME_EXTRA);
+        stack[stackPtr] = function;
+        stack[stackPtr + 1] = first;
+        stack[stackPtr + 2] = second;
+        stack[stackPtr + 3] = third;
+        stack[stackPtr + 4] = fourth;
+        stack[stackPtr + 5] = STACK_MARKER_4;
+        stackPtr += 4 + STACK_FRAME_EXTRA;
         try {
             return function.execute(first, second, third, fourth);
         }
         finally {
-            popStackFrame();
+            popStackFrame(4);
         }
     }
 
@@ -689,12 +881,20 @@ public final class LispThread extends LispObject
                               LispObject second, LispObject third,
                               LispObject fourth, LispObject fifth)
     {
-        pushStackFrame(new LispStackFrame(function, first, second, third, fourth, fifth));
+        ensureStackCapacity(5 + STACK_FRAME_EXTRA);
+        stack[stackPtr] = function;
+        stack[stackPtr + 1] = first;
+        stack[stackPtr + 2] = second;
+        stack[stackPtr + 3] = third;
+        stack[stackPtr + 4] = fourth;
+        stack[stackPtr + 5] = fifth;
+        stack[stackPtr + 6] = STACK_MARKER_5;
+        stackPtr += 5 + STACK_FRAME_EXTRA;
         try {
             return function.execute(first, second, third, fourth, fifth);
         }
         finally {
-            popStackFrame();
+            popStackFrame(5);
         }
     }
 
@@ -704,13 +904,21 @@ public final class LispThread extends LispObject
                               LispObject fourth, LispObject fifth,
                               LispObject sixth)
     {
-        pushStackFrame(new LispStackFrame(function, first, second, 
-					  third, fourth, fifth, sixth));
+        ensureStackCapacity(6 + STACK_FRAME_EXTRA);
+        stack[stackPtr] = function;
+        stack[stackPtr + 1] = first;
+        stack[stackPtr + 2] = second;
+        stack[stackPtr + 3] = third;
+        stack[stackPtr + 4] = fourth;
+        stack[stackPtr + 5] = fifth;
+        stack[stackPtr + 6] = sixth;
+        stack[stackPtr + 7] = STACK_MARKER_6;
+        stackPtr += 6 + STACK_FRAME_EXTRA;
         try {
             return function.execute(first, second, third, fourth, fifth, sixth);
         }
         finally {
-            popStackFrame();
+            popStackFrame(6);
         }
     }
 
@@ -720,14 +928,23 @@ public final class LispThread extends LispObject
                               LispObject fourth, LispObject fifth,
                               LispObject sixth, LispObject seventh)
     {
-        pushStackFrame(new LispStackFrame(function, first, second, third, 
-					  fourth, fifth, sixth, seventh));
+        ensureStackCapacity(7 + STACK_FRAME_EXTRA);
+        stack[stackPtr] = function;
+        stack[stackPtr + 1] = first;
+        stack[stackPtr + 2] = second;
+        stack[stackPtr + 3] = third;
+        stack[stackPtr + 4] = fourth;
+        stack[stackPtr + 5] = fifth;
+        stack[stackPtr + 6] = sixth;
+        stack[stackPtr + 7] = seventh;
+        stack[stackPtr + 8] = STACK_MARKER_7;
+        stackPtr += 7 + STACK_FRAME_EXTRA;
         try {
             return function.execute(first, second, third, fourth, fifth, sixth,
                                     seventh);
         }
         finally {
-            popStackFrame();
+            popStackFrame(7);
         }
     }
 
@@ -737,25 +954,39 @@ public final class LispThread extends LispObject
                               LispObject sixth, LispObject seventh,
                               LispObject eighth)
     {
-        pushStackFrame(new LispStackFrame(function, first, second, third, 
-					  fourth, fifth, sixth, seventh, eighth));
+        ensureStackCapacity(8 + STACK_FRAME_EXTRA);
+        stack[stackPtr] = function;
+        stack[stackPtr + 1] = first;
+        stack[stackPtr + 2] = second;
+        stack[stackPtr + 3] = third;
+        stack[stackPtr + 4] = fourth;
+        stack[stackPtr + 5] = fifth;
+        stack[stackPtr + 6] = sixth;
+        stack[stackPtr + 7] = seventh;
+        stack[stackPtr + 8] = eighth;
+        stack[stackPtr + 9] = STACK_MARKER_8;
+        stackPtr += 8 + STACK_FRAME_EXTRA;
         try {
             return function.execute(first, second, third, fourth, fifth, sixth,
                                     seventh, eighth);
         }
         finally {
-            popStackFrame();
+            popStackFrame(8);
         }
     }
 
     public LispObject execute(LispObject function, LispObject[] args)
     {
-        pushStackFrame(new LispStackFrame(function, args));
+        ensureStackCapacity(args.length + STACK_FRAME_EXTRA);
+        stack[stackPtr] = function;
+        System.arraycopy(args, 0, stack, stackPtr + 1, args.length);
+        stack[stackPtr + args.length + 1] = new StackMarker(args.length);
+        stackPtr += args.length + STACK_FRAME_EXTRA;
         try {
             return function.execute(args);
         }
         finally {
-            popStackFrame();
+            popStackFrame(args.length);
         }
     }
 
@@ -766,14 +997,15 @@ public final class LispThread extends LispObject
 
     public void printBacktrace(int limit)
     {
-        if (stack != null) {
+        StackFrame stackTop = getStackTop();
+        if (stackTop != null) {
             int count = 0;
             Stream out =
                 checkCharacterOutputStream(Symbol.TRACE_OUTPUT.symbolValue());
             out._writeLine("Evaluation stack:");
             out._finishOutput();
 
-            StackFrame s = stack;
+            StackFrame s = stackTop;
             while (s != null) {
                 out._writeString("  ");
                 out._writeString(String.valueOf(count));
@@ -791,10 +1023,11 @@ public final class LispThread extends LispObject
 
     public LispObject backtrace(int limit)
     {
+        StackFrame stackTop = getStackTop();
         LispObject result = NIL;
-        if (stack != null) {
+        if (stackTop != null) {
             int count = 0;
-            StackFrame s = stack;
+            StackFrame s = stackTop;
             while (s != null) {
                 result = result.push(s);
                 if (limit > 0 && ++count == limit)
@@ -807,28 +1040,34 @@ public final class LispThread extends LispObject
 
     public void incrementCallCounts()
     {
-        StackFrame s = stack;
-
-        for (int i = 0; i < 8; i++) {
-            if (s == null)
-                break;
-	    if (s instanceof LispStackFrame) {
-		LispObject operator = ((LispStackFrame)s).getOperator();
-		if (operator != null) {
-		    operator.incrementHotCount();
-		    operator.incrementCallCount();
-		}
-		s = s.getNext();
-	    }
-        }
-
-        while (s != null) {
-	    if (s instanceof LispStackFrame) {
-		LispObject operator = ((LispStackFrame)s).getOperator();
-		if (operator != null)
-		    operator.incrementCallCount();
-	    }
-	    s = s.getNext();
+        topStackSegment.stackPtr = stackPtr;
+        int depth = 0;
+        for (StackSegment segment = topStackSegment; segment != null; segment = segment.next) {
+            Object[] stk = segment.stack;
+            int framePos = segment.stackPtr;
+            while (framePos > 0) {
+                depth++;
+                Object stackObj = stk[framePos - 1];
+                int numArgs;
+                if (stackObj instanceof StackMarker) {
+                    numArgs = ((StackMarker) stackObj).getNumArgs();
+                } else if (stackObj instanceof LispStackFrame) {
+                    numArgs = ((LispStackFrame) stackObj).getNumArgs();
+                } else {
+                    assert stackObj instanceof JavaStackFrame;
+                    framePos--;
+                    continue;
+                }
+                // lisp stack frame
+                framePos -= numArgs + STACK_FRAME_EXTRA;
+                LispObject operator = (LispObject) stack[framePos];
+                if (operator != null) {
+                    if (depth <= 8) {
+                        operator.incrementHotCount();
+                    }
+                    operator.incrementCallCount();
+                }
+            }
         }
     }
 
@@ -911,14 +1150,14 @@ public final class LispThread extends LispObject
             LispObject name = NIL;
             if (length > 1) {
                 if ((length - 1) % 2 != 0)
-                    error(new ProgramError("Odd number of keyword arguments."));
+                    program_error("Odd number of keyword arguments.");
                 if (length > 3)
                     error(new WrongNumberOfArgumentsException(this, -1, 2)); // don't count the keyword itself as an argument
                 if (args[1] == Keyword.NAME)
                     name = args[2].STRING();
                 else
-                    error(new ProgramError("Unrecognized keyword argument " +
-                                            args[1].princToString() + "."));
+                    program_error("Unrecognized keyword argument "
+                                  + args[1].princToString() + ".");
             }
             return new LispThread(checkFunction(args[0]), name);
         }
@@ -940,7 +1179,7 @@ public final class LispThread extends LispObject
     doc="Returns T if THREAD is alive.")
     private static final Primitive THREAD_ALIVE_P =
         new Primitive("thread-alive-p", PACKAGE_THREADS, true, "thread",
-		      "Boolean predicate whether THREAD is alive.")
+              "Boolean predicate whether THREAD is alive.")
     {
         @Override
         public LispObject execute(LispObject arg)
@@ -998,35 +1237,59 @@ public final class LispThread extends LispObject
             } 
         }
     };
+    
+    final static DoubleFloat THOUSAND = new DoubleFloat(1000);
 
-
-    public static final long javaSleepInterval(LispObject lispSleep)
-
-    {
-        double d =
-            checkDoubleFloat(lispSleep.multiplyBy(new DoubleFloat(1000))).getValue();
-        if (d < 0)
-            type_error(lispSleep, list(Symbol.REAL, Fixnum.ZERO));
-
-        return (d < Long.MAX_VALUE ? (long) d : Long.MAX_VALUE);
+    static final long sleepMillisPart(LispObject seconds) {
+      double d 
+        = checkDoubleFloat(seconds.multiplyBy(THOUSAND)).getValue();
+      if (d < 0) {
+        type_error(seconds, list(Symbol.REAL, Fixnum.ZERO));
+      }
+      return (d < Long.MAX_VALUE ? (long) d : Long.MAX_VALUE);
     }
 
+    static final int sleepNanosPart(LispObject seconds) {
+      double d  // d contains millis
+        = checkDoubleFloat(seconds.multiplyBy(THOUSAND)).getValue();
+      double n = d * 1000000; // sleep interval in nanoseconds
+      d = 1.0e6 * ((long)d); //  sleep interval to millisecond precision
+      n = n - d; 
+
+      return (n < Integer.MAX_VALUE ? (int) n : Integer.MAX_VALUE);
+    }
+
+
     @DocString(name="sleep", args="seconds",
-    doc="Causes the invoking thread to sleep for SECONDS seconds.\n"+
-        "SECONDS may be a value between 0 1and 1.")
+    doc="Causes the invoking thread to sleep for an interveral expressed in SECONDS.\n"
+      + "SECONDS may be specified as a fraction of a second, with intervals\n"
+      + "less than or equal to a nanosecond resulting in a yield of execution\n"
+      + "to other waiting threads rather than an actual sleep.\n"
+      + "A zero value of SECONDS *may* result in the JVM sleeping indefinitely,\n"
+      + "depending on the implementation.")
     private static final Primitive SLEEP = new Primitive("sleep", PACKAGE_CL, true)
     {
         @Override
         public LispObject execute(LispObject arg)
         {
+          long millis = sleepMillisPart(arg);
+          int nanos = sleepNanosPart(arg);
+          boolean zeroArgP = arg.ZEROP() != NIL;
 
-            try {
-                Thread.sleep(javaSleepInterval(arg));
-            }
-            catch (InterruptedException e) {
-                currentThread().processThreadInterrupts();
-            }
-            return NIL;
+          try {
+            if (millis == 0 && nanos == 0) { 
+              if (zeroArgP) {
+                Thread.sleep(0, 0);
+              } else { 
+                Thread.sleep(0, 1);
+              }
+            } else {
+              Thread.sleep(millis, nanos);
+            } 
+          } catch (InterruptedException e) {
+            currentThread().processThreadInterrupts();
+          }
+          return NIL;
         }
     };
 
@@ -1078,8 +1341,8 @@ public final class LispThread extends LispObject
         "order is not guaranteed.")
     private static final Primitive INTERRUPT_THREAD =
         new Primitive("interrupt-thread", PACKAGE_THREADS, true,
-		      "thread function &rest args",
-		      "Interrupts THREAD and forces it to apply FUNCTION to ARGS.\nWhen the function returns, the thread's original computation continues. If  multiple interrupts are queued for a thread, they are all run, but the order is not guaranteed.")
+              "thread function &rest args",
+              "Interrupts THREAD and forces it to apply FUNCTION to ARGS.\nWhen the function returns, the thread's original computation continues. If  multiple interrupts are queued for a thread, they are all run, but the order is not guaranteed.")
     {
         @Override
         public LispObject execute(LispObject[] args)
@@ -1102,156 +1365,186 @@ public final class LispThread extends LispObject
         }
     };
 
+    public static final Primitive CURRENT_THREAD 
+      = new pf_current_thread();
     @DocString(name="current-thread",
-    doc="Returns a reference to invoking thread.")
-    private static final Primitive CURRENT_THREAD =
-        new Primitive("current-thread", PACKAGE_THREADS, true)
-    {
-        @Override
-        public LispObject execute()
-        {
-            return currentThread();
-        }
+               doc="Returns a reference to invoking thread.")
+    private static final class pf_current_thread extends Primitive {
+      pf_current_thread() {
+        super("current-thread", PACKAGE_THREADS, true);
+      }
+      @Override
+      public LispObject execute() {
+        return currentThread();
+      }
     };
 
+    public static final Primitive BACKTRACE
+      = new pf_backtrace();
     @DocString(name="backtrace",
-               doc="Returns a backtrace of the invoking thread.")
-    private static final Primitive BACKTRACE =
-        new Primitive("backtrace", PACKAGE_SYS, true)
-    {
-        @Override
-        public LispObject execute(LispObject[] args)
-
-        {
-            if (args.length > 1)
-                return error(new WrongNumberOfArgumentsException(this, -1, 1));
-            int limit = args.length > 0 ? Fixnum.getValue(args[0]) : 0;
-            return currentThread().backtrace(limit);
-        }
-    };
-    @DocString(name="frame-to-string", args="frame")
-    private static final Primitive FRAME_TO_STRING =
-        new Primitive("frame-to-string", PACKAGE_SYS, true)
-    {
-        @Override
-        public LispObject execute(LispObject[] args)
-
-        {
-            if (args.length != 1)
-                return error(new WrongNumberOfArgumentsException(this, 1));
-            
-            return checkStackFrame(args[0]).toLispString();
-        }
+               doc="Returns a Java backtrace of the invoking thread.")
+    private static final class pf_backtrace extends Primitive {
+      pf_backtrace() {
+        super("backtrace", PACKAGE_SYS, true);
+      }
+      @Override
+      public LispObject execute(LispObject[] args) {
+        if (args.length > 1)
+          return error(new WrongNumberOfArgumentsException(this, -1, 1));
+        int limit = args.length > 0 ? Fixnum.getValue(args[0]) : 0;
+        return currentThread().backtrace(limit);
+      }
     };
 
+    public static final Primitive FRAME_TO_STRING
+      = new pf_frame_to_string();
+    @DocString(name="frame-to-string", 
+               args="frame",
+               doc="Convert stack FRAME to a (potentially) readable string.")
+    private static final class pf_frame_to_string extends Primitive {
+      pf_frame_to_string() {
+        super("frame-to-string", PACKAGE_SYS, true);
+      }
+      @Override
+      public LispObject execute(LispObject[] args) {
+        if (args.length != 1)
+          return error(new WrongNumberOfArgumentsException(this, 1));
+        return checkStackFrame(args[0]).toLispString();
+      }
+    };
+
+    public static final Primitive FRAME_TO_LIST
+      = new pf_frame_to_list();
     @DocString(name="frame-to-list", args="frame")
-    private static final Primitive FRAME_TO_LIST =
-        new Primitive("frame-to-list", PACKAGE_SYS, true)
-    {
-        @Override
-        public LispObject execute(LispObject[] args)
+    private static final class pf_frame_to_list extends Primitive {
+      pf_frame_to_list() {
+        super("frame-to-list", PACKAGE_SYS, true);
+      }
+      @Override
+      public LispObject execute(LispObject[] args) {
+        if (args.length != 1)
+          return error(new WrongNumberOfArgumentsException(this, 1));
 
-        {
-            if (args.length != 1)
-                return error(new WrongNumberOfArgumentsException(this, 1));
-
-            return checkStackFrame(args[0]).toLispList();
-        }
+        return checkStackFrame(args[0]).toLispList();
+      }
     };
 
 
+    public static final SpecialOperator SYNCHRONIZED_ON 
+      = new so_synchronized_on();
     @DocString(name="synchronized-on", args="form &body body")
-    private static final SpecialOperator SYNCHRONIZED_ON =
-        new SpecialOperator("synchronized-on", PACKAGE_THREADS, true,
-                            "form &body body")
-    {
-        @Override
-        public LispObject execute(LispObject args, Environment env)
+    private static final class so_synchronized_on extends SpecialOperator {
+      so_synchronized_on() {
+        super("synchronized-on", PACKAGE_THREADS, true, "form &body body");
+      }
+      @Override
+      public LispObject execute(LispObject args, Environment env) {
+        if (args == NIL)
+          return error(new WrongNumberOfArgumentsException(this, 1));
+        
+        LispThread thread = LispThread.currentThread();
+        synchronized (eval(args.car(), env, thread).lockableInstance()) {
+          return progn(args.cdr(), env, thread);
+        }
+      }
+    };
 
-        {
-          if (args == NIL)
-            return error(new WrongNumberOfArgumentsException(this, 1));
+  
+    public static final Primitive OBJECT_WAIT
+      = new pf_object_wait();
+    @DocString(
+    name="object-wait", args="object &optional timeout", 
+    doc="Causes the current thread to block until object-notify or object-notify-all is called on OBJECT.\n"
+       + "Optionally unblock execution after TIMEOUT seconds.  A TIMEOUT of zero\n"
+       + "means to wait indefinitely.\n"
+       + "A non-zero TIMEOUT of less than a nanosecond is interpolated as a nanosecond wait."
+       + "\n"
+       + "See the documentation of java.lang.Object.wait() for further\n"
+       + "information.\n"
+    )
+    private static final class pf_object_wait extends Primitive {
+      pf_object_wait() {
+        super("object-wait", PACKAGE_THREADS, true);
+      }
+      @Override
+      public LispObject execute(LispObject object) {
+        try {
+          object.lockableInstance().wait();
+        } catch (InterruptedException e) {
+          currentThread().processThreadInterrupts();
+        } catch (IllegalMonitorStateException e) {
+          return error(new IllegalMonitorState(e.getMessage()));
+        }
+        return NIL;
+      }
 
-          LispThread thread = LispThread.currentThread();
-          synchronized (eval(args.car(), env, thread).lockableInstance()) {
-              return progn(args.cdr(), env, thread);
+      @Override
+      public LispObject execute(LispObject object, LispObject timeout) {
+        long millis = sleepMillisPart(timeout);
+        int nanos = sleepNanosPart(timeout);
+        boolean zeroArgP = timeout.ZEROP() != NIL;
+          
+        try {
+          if (millis == 0 && nanos == 0) { 
+            if (zeroArgP) {
+              object.lockableInstance().wait(0, 0);
+            } else {
+              object.lockableInstance().wait(0, 1);
+            }
+          } else {
+            object.lockableInstance().wait(millis, nanos);
           }
+        } catch (InterruptedException e) {
+          currentThread().processThreadInterrupts();
+        } catch (IllegalMonitorStateException e) {
+          return error(new IllegalMonitorState(e.getMessage()));
         }
+        return NIL;
+      }
     };
 
-    @DocString(name="object-wait", args="object &optional timeout")
-    private static final Primitive OBJECT_WAIT =
-        new Primitive("object-wait", PACKAGE_THREADS, true)
-    {
-        @Override
-        public LispObject execute(LispObject object)
-
-        {
-            try {
-                object.lockableInstance().wait();
-            }
-            catch (InterruptedException e) {
-                currentThread().processThreadInterrupts();
-            }
-            catch (IllegalMonitorStateException e) {
-                return error(new IllegalMonitorState(e.getMessage()));
-            }
-            return NIL;
-        }
-
-        @Override
-        public LispObject execute(LispObject object, LispObject timeout)
-
-        {
-            try {
-                object.lockableInstance().wait(javaSleepInterval(timeout));
-            }
-            catch (InterruptedException e) {
-                currentThread().processThreadInterrupts();
-            }
-            catch (IllegalMonitorStateException e) {
-                return error(new IllegalMonitorState(e.getMessage()));
-            }
-            return NIL;
+    public static final Primitive OBJECT_NOTIFY
+      = new pf_object_notify();
+    @DocString(name="object-notify", 
+               args="object",
+               doc="Wakes up a single thread that is waiting on OBJECT's monitor."
++ "\nIf any threads are waiting on this object, one of them is chosen to be"
++ " awakened. The choice is arbitrary and occurs at the discretion of the"
++ " implementation. A thread waits on an object's monitor by calling one"
++ " of the wait methods.")
+    private static final class pf_object_notify extends Primitive {
+      pf_object_notify() {
+        super("object-notify", PACKAGE_THREADS, true, "object");
+      }
+      @Override
+      public LispObject execute(LispObject object) {
+        try {
+          object.lockableInstance().notify();
+        } catch (IllegalMonitorStateException e) {
+          return error(new IllegalMonitorState(e.getMessage()));
         }
+        return NIL;
+      }
     };
 
-    @DocString(name="object-notify", args="object")
-    private static final Primitive OBJECT_NOTIFY =
-        new Primitive("object-notify", PACKAGE_THREADS, true,
-                      "object")
-    {
-        @Override
-        public LispObject execute(LispObject object)
-
-        {
-            try {
-                object.lockableInstance().notify();
-            }
-            catch (IllegalMonitorStateException e) {
-                return error(new IllegalMonitorState(e.getMessage()));
-            }
-            return NIL;
+    public static final Primitive OBJECT_NOTIFY_ALL
+      = new pf_object_notify_all();
+    @DocString(name="object-notify-all", 
+               args="object",
+               doc="Wakes up all threads that are waiting on this OBJECT's monitor."
++ "\nA thread waits on an object's monitor by calling one of the wait methods.")
+    private static final class pf_object_notify_all extends Primitive {
+      pf_object_notify_all() {
+        super("object-notify-all", PACKAGE_THREADS, true);
+      }
+      @Override
+      public LispObject execute(LispObject object) {
+        try {
+          object.lockableInstance().notifyAll();
+        } catch (IllegalMonitorStateException e) {
+          return error(new IllegalMonitorState(e.getMessage()));
         }
+        return NIL;
+      }
     };
-
-    @DocString(name="object-notify-all", args="object")
-    private static final Primitive OBJECT_NOTIFY_ALL =
-        new Primitive("object-notify-all", PACKAGE_THREADS, true)
-    {
-        @Override
-        public LispObject execute(LispObject object)
-
-        {
-            try {
-                object.lockableInstance().notifyAll();
-            }
-            catch (IllegalMonitorStateException e) {
-                return error(new IllegalMonitorState(e.getMessage()));
-            }
-            return NIL;
-        }
-    };
-
-
 }
