@@ -2,7 +2,7 @@
  * Package.java
  *
  * Copyright (C) 2002-2007 Peter Graves <peter@armedbear.org>
- * $Id: Package.java 14061 2012-08-06 07:45:59Z ehuelsmann $
+ * $Id: Package.java 14570 2013-07-22 13:21:06Z mevenson $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,6 +38,7 @@ import static org.armedbear.lisp.Lisp.*;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
+import java.util.Map;
 import java.util.Iterator;
 import java.util.List;
 import java.util.concurrent.ConcurrentHashMap;
@@ -63,6 +64,7 @@ public final class Package extends LispObject implements java.io.Serializable
     private transient ArrayList<String> nicknames;
     private transient LispObject useList = null;
     private transient ArrayList<Package> usedByList = null;
+    private transient ConcurrentHashMap<String, Package> localNicknames;
 
     // Anonymous package.
     public Package()
@@ -176,6 +178,13 @@ public final class Package extends LispObject implements java.io.Serializable
               }
             }
 
+            LispObject packages = Packages.getPackagesNicknamingPackage(this);
+            while (packages != NIL) {
+              Package p = (Package)((Cons)packages).car();
+              packages = ((Cons)packages).cdr();
+              p.removeLocalPackageNicknamesForPackage(this);
+            }
+
             Packages.deletePackage(this);
 
             makeSymbolsUninterned(internalSymbols);
@@ -459,13 +468,18 @@ public final class Package extends LispObject implements java.io.Serializable
             }
         }
         // Reaching here, it's OK to remove the symbol.
-        if (internalSymbols.get(symbol.name.toString()) == symbol)
-            internalSymbols.remove(symbol.name.toString());
-        else if (externalSymbols.get(symbol.name.toString()) == symbol)
+        boolean found = false;
+        if (externalSymbols.get(symbol.name.toString()) == symbol) {
             externalSymbols.remove(symbol.name.toString());
-        else
-            // Not found.
+            found = true;
+        }
+        if (internalSymbols.get(symbol.name.toString()) == symbol) {
+            internalSymbols.remove(symbol.name.toString());
+            found = true;
+        }
+        if (! found)
             return NIL;
+
         if (shadow) {
             Debug.assertTrue(shadowingSymbols != null);
             shadowingSymbols.remove(symbolName);
@@ -551,28 +565,16 @@ public final class Package extends LispObject implements java.io.Serializable
     public synchronized void unexport(final Symbol symbol)
 
     {
-        if (symbol.getPackage() == this) {
-            if (externalSymbols.get(symbol.name.toString()) == symbol) {
-                externalSymbols.remove(symbol.name.toString());
-                internalSymbols.put(symbol.name.toString(), symbol);
-            }
-        } else {
-            // Signal an error if symbol is not accessible.
-            if (useList instanceof Cons) {
-                LispObject usedPackages = useList;
-                while (usedPackages != NIL) {
-                    Package pkg = (Package) usedPackages.car();
-                    if (pkg.findExternalSymbol(symbol.name) == symbol)
-                        return; // OK.
-                    usedPackages = usedPackages.cdr();
-                }
-            }
-            StringBuilder sb = new StringBuilder("The symbol ");
-            sb.append(symbol.getQualifiedName());
-            sb.append(" is not accessible in package ");
-            sb.append(name);
-            error(new PackageError(sb.toString()));
-        }
+      if (externalSymbols.get(symbol.name.toString()) == symbol) {
+        externalSymbols.remove(symbol.name.toString());
+        internalSymbols.put(symbol.name.toString(), symbol);
+      } else if (findAccessibleSymbol(symbol.name.toString()) != symbol) {
+        StringBuilder sb = new StringBuilder("The symbol ");
+        sb.append(symbol.getQualifiedName());
+        sb.append(" is not accessible in package ");
+        sb.append(name);
+        error(new PackageError(sb.toString()));
+      }
     }
 
     public synchronized void shadow(final String symbolName)
@@ -600,49 +602,27 @@ public final class Package extends LispObject implements java.io.Serializable
 
     public synchronized void shadowingImport(Symbol symbol)
     {
-        LispObject where = NIL;
         final String symbolName = symbol.getName();
-        Symbol sym = externalSymbols.get(symbol.name.toString());
-        if (sym != null) {
-            where = Keyword.EXTERNAL;
-        } else {
+        Symbol sym = externalSymbols.get(symbolName);
+        if (sym == null)
             sym = internalSymbols.get(symbol.name.toString());
-            if (sym != null) {
-                where = Keyword.INTERNAL;
-            } else {
-                // Look in external symbols of used packages.
-                if (useList instanceof Cons) {
-                    LispObject usedPackages = useList;
-                    while (usedPackages != NIL) {
-                        Package pkg = (Package) usedPackages.car();
-                        sym = pkg.findExternalSymbol(symbol.name);
-                        if (sym != null) {
-                            where = Keyword.INHERITED;
-                            break;
-                        }
-                        usedPackages = usedPackages.cdr();
-                    }
-                }
-            }
+
+        // if a different symbol with the same name is accessible,
+        // [..] which implies that it must be uninterned if it was present
+        if (sym != null && sym != symbol) {
+            if (shadowingSymbols != null)
+                shadowingSymbols.remove(symbolName);
+            unintern(sym);
         }
-        if (sym != null) {
-            if (where == Keyword.INTERNAL || where == Keyword.EXTERNAL) {
-                if (sym != symbol) {
-                    if (shadowingSymbols != null)
-                        shadowingSymbols.remove(symbolName);
-                    unintern(sym);
-                } else if (where == Keyword.INTERNAL) {
-                    // Assert rgument is already correctly a shadowing import
-                    Debug.assertTrue(shadowingSymbols != null);
-                    Debug.assertTrue(shadowingSymbols.get(symbolName) != null);
-                    return;
-                }
-            }
+
+        if (sym == null || sym != symbol) {
+            // there was no symbol, or we just unintered it another one
+            // intern the new one
+            internalSymbols.put(symbol.name.toString(), symbol);
         }
-        internalSymbols.put(symbol.name.toString(), symbol);
+
         if (shadowingSymbols == null)
             shadowingSymbols = new HashMap<String,Symbol>();
-        Debug.assertTrue(shadowingSymbols.get(symbolName) == null);
         shadowingSymbols.put(symbolName, symbol);
     }
 
@@ -758,6 +738,76 @@ public final class Package extends LispObject implements java.io.Serializable
         return list;
     }
 
+  public LispObject getLocalPackageNicknames()
+  {
+    LispObject list = NIL;
+    if (localNicknames != null) {
+      for (Map.Entry<String, Package> entry : localNicknames.entrySet()) {
+        list = new Cons(new Cons(entry.getKey(), entry.getValue()), list);
+      }
+    }
+    return list;
+  }
+
+  public LispObject addLocalPackageNickname(String name, Package pack)
+  {
+    if (localNicknames == null) {
+      localNicknames = new ConcurrentHashMap<String, Package>();
+    }
+    if (localNicknames.containsKey(name)) {
+      if (localNicknames.get(name) != pack) {
+        return error(new LispError(name + " is already a nickname for "
+                                   + pack.getName()));
+      } else {
+        // nothing to do
+        return this;
+      }
+    } else {
+      localNicknames.put(name, pack);
+      return this;
+    }
+  }
+
+  public LispObject removeLocalPackageNickname(String name)
+  {
+    if (localNicknames == null || !localNicknames.containsKey(name)) {
+      return NIL;
+    } else {
+      // return generalized boolean: package that was nicknamed to `name'
+      return localNicknames.remove(name);
+    }
+  }
+
+  public void removeLocalPackageNicknamesForPackage(Package p)
+  {
+    if (localNicknames == null || !localNicknames.containsValue(p)) {
+      return;
+    } else {
+      for (Map.Entry<String, Package> entry : localNicknames.entrySet()) {
+        if (entry.getValue() == p) {
+          localNicknames.remove(entry.getKey());
+        }
+      }
+    }
+  }
+
+  public Collection<Package> getLocallyNicknamedPackages()
+  {
+    // for implementing package-locally-nicknamed-by-list
+    if (localNicknames == null) return new ArrayList<Package>();
+    else return localNicknames.values();
+  }
+
+  // Find package named `name', taking local nicknames into account
+  public Package findPackage(String name)
+  {
+    if (localNicknames != null) {
+      Package pkg = localNicknames.get(name);
+      if (pkg != null) return pkg;
+    }
+    return Packages.findPackageGlobally(name);
+  }
+
     public LispObject getShadowingSymbols()
     {
         LispObject list = NIL;
@@ -878,7 +928,7 @@ public final class Package extends LispObject implements java.io.Serializable
     }
 
     public Object readResolve() throws java.io.ObjectStreamException {
-	Package pkg = Packages.findPackage(name);
+	Package pkg = findPackage(name);
 	if(pkg != null) {
 	    return pkg;
 	} else {
