@@ -2,7 +2,7 @@
  * StandardClass.java
  *
  * Copyright (C) 2003-2005 Peter Graves
- * $Id: StandardClass.java 14083 2012-08-14 11:53:16Z rschlatte $
+ * $Id: StandardClass.java 14501 2013-05-15 08:06:06Z rschlatte $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,27 +37,20 @@ import static org.armedbear.lisp.Lisp.*;
 
 public class StandardClass extends SlotClass
 {
-
-  public static Symbol symName = PACKAGE_MOP.intern("NAME");
-  public static Symbol symLayout = PACKAGE_MOP.intern("LAYOUT");
-  public static Symbol symDirectSuperclasses
-    = PACKAGE_MOP.intern("DIRECT-SUPERCLASSES");
-  public static Symbol symDirectSubclasses
-    = PACKAGE_MOP.intern("DIRECT-SUBCLASSES");
-  public static Symbol symPrecedenceList
-    = PACKAGE_MOP.intern("PRECEDENCE-LIST");
-  public static Symbol symDirectMethods
-    = PACKAGE_MOP.intern("DIRECT-METHODS");
-  public static Symbol symDirectSlots
-    = PACKAGE_MOP.intern("DIRECT-SLOTS");
-  public static Symbol symSlots
-    = PACKAGE_MOP.intern("SLOTS");
+  // Slot names for standard-class.  Must agree with
+  // redefine-class-forwarder calls in clos.lisp.
+  public static Symbol symName = Symbol.NAME;
+  public static Symbol symLayout = Symbol.LAYOUT;
+  public static Symbol symDirectSuperclasses = Symbol.DIRECT_SUPERCLASSES;
+  public static Symbol symDirectSubclasses = Symbol.DIRECT_SUBCLASSES;
+  public static Symbol symPrecedenceList = Symbol.PRECEDENCE_LIST;
+  public static Symbol symDirectMethods = Symbol.DIRECT_METHODS;
+  public static Symbol symDirectSlots = Symbol.DIRECT_SLOTS; 
+  public static Symbol symSlots = Symbol.SLOTS;
   public static Symbol symDirectDefaultInitargs
-    = PACKAGE_MOP.intern("DIRECT-DEFAULT-INITARGS");
-  public static Symbol symDefaultInitargs
-    = PACKAGE_MOP.intern("DEFAULT-INITARGS");
-  public static Symbol symFinalizedP
-    = PACKAGE_MOP.intern("FINALIZED-P");
+    = Symbol.DIRECT_DEFAULT_INITARGS;
+  public static Symbol symDefaultInitargs = Symbol.DEFAULT_INITARGS;
+  public static Symbol symFinalizedP = Symbol.FINALIZED_P;
 
   // used as init-function for slots in this file.
   static Function constantlyNil = new Function() {
@@ -116,8 +109,6 @@ public class StandardClass extends SlotClass
         }
       };
 
-  
-
   public StandardClass()
   {
       super(layoutStandardClass);
@@ -373,20 +364,6 @@ public class StandardClass extends SlotClass
     return super.typep(type);
   }
 
-  public LispObject allocateInstance()
-  {
-    Layout layout = getClassLayout();
-    if (layout == null)
-      {
-        Symbol.ERROR.execute(Symbol.SIMPLE_ERROR,
-                             Keyword.FORMAT_CONTROL,
-                             new SimpleString("No layout for class ~S."),
-                             Keyword.FORMAT_ARGUMENTS,
-                             list(this));
-      }
-    return new StandardObject(this, layout.getLength());
-  }
-
   @Override
   public String printObject()
   {
@@ -403,31 +380,20 @@ public class StandardClass extends SlotClass
   private static final LispObject standardClassSlotDefinitions()
   {
     return
-        list(helperMakeSlotDefinition("NAME", constantlyNil),
-             helperMakeSlotDefinition("LAYOUT", constantlyNil),
-             helperMakeSlotDefinition("DIRECT-SUPERCLASSES", constantlyNil),
-             helperMakeSlotDefinition("DIRECT-SUBCLASSES", constantlyNil),
-             helperMakeSlotDefinition("PRECEDENCE-LIST", constantlyNil),
-             helperMakeSlotDefinition("DIRECT-METHODS", constantlyNil),
-             helperMakeSlotDefinition("DIRECT-SLOTS", constantlyNil),
-             helperMakeSlotDefinition("SLOTS", constantlyNil),
-             helperMakeSlotDefinition("DIRECT-DEFAULT-INITARGS", constantlyNil),
-             helperMakeSlotDefinition("DEFAULT-INITARGS", constantlyNil),
-             helperMakeSlotDefinition("FINALIZED-P", constantlyNil),
-             new SlotDefinition(Symbol._DOCUMENTATION,
-                                list(PACKAGE_MOP.intern("CLASS-DOCUMENTATION")),
-                                constantlyNil, list(internKeyword("DOCUMENTATION"))));
-  }
-
-
-
-  private static final SlotDefinition helperMakeSlotDefinition(String name,
-                                                               Function init)
-  {
-    return
-        new SlotDefinition(PACKAGE_MOP.intern(name),   // name
-             list(PACKAGE_MOP.intern("CLASS-" + name)), // readers
-             init);
+      list(new SlotDefinition(symName, list(Symbol.CLASS_NAME), constantlyNil),
+           new SlotDefinition(symLayout, list(Symbol.CLASS_LAYOUT), constantlyNil),
+           new SlotDefinition(symDirectSuperclasses, list(Symbol.CLASS_DIRECT_SUPERCLASSES), constantlyNil),
+           new SlotDefinition(symDirectSubclasses, list(Symbol.CLASS_DIRECT_SUBCLASSES), constantlyNil),
+           new SlotDefinition(symPrecedenceList, list(Symbol.CLASS_PRECEDENCE_LIST), constantlyNil),
+           new SlotDefinition(symDirectMethods, list(Symbol.CLASS_DIRECT_METHODS), constantlyNil),
+           new SlotDefinition(symDirectSlots, list(Symbol.CLASS_DIRECT_SLOTS), constantlyNil),
+           new SlotDefinition(symSlots, list(Symbol.CLASS_SLOTS), constantlyNil),
+           new SlotDefinition(symDirectDefaultInitargs, list(Symbol.CLASS_DIRECT_DEFAULT_INITARGS), constantlyNil),
+           new SlotDefinition(symDefaultInitargs, list(Symbol.CLASS_DEFAULT_INITARGS), constantlyNil),
+           new SlotDefinition(symFinalizedP, list(Symbol.CLASS_FINALIZED_P), constantlyNil),
+           new SlotDefinition(Symbol._DOCUMENTATION,
+                              list(Symbol.CLASS_DOCUMENTATION),
+                              constantlyNil, list(internKeyword("DOCUMENTATION"))));
   }
 
   private static final StandardClass addStandardClass(Symbol name,
@@ -438,6 +404,14 @@ public class StandardClass extends SlotClass
     return c;
   }
 
+  private static final FuncallableStandardClass addFuncallableStandardClass
+    (Symbol name, LispObject directSuperclasses)
+  {
+    FuncallableStandardClass c = new FuncallableStandardClass(name, directSuperclasses);
+    addClass(name, c);
+    return c;
+  }
+
   // At this point, BuiltInClass.java has not been completely loaded yet, and
   // BuiltInClass.CLASS_T is null. So we need to call setDirectSuperclass()
   // for STANDARD_CLASS and STANDARD_OBJECT in initializeStandardClasses()
@@ -480,29 +454,18 @@ public class StandardClass extends SlotClass
 
 
   // BuiltInClass.FUNCTION is also null here (see previous comment).
+  // Following SBCL's lead, we make funcallable-standard-object a
+  // funcallable-standard-class.
   public static final StandardClass FUNCALLABLE_STANDARD_OBJECT =
-      addStandardClass(Symbol.FUNCALLABLE_STANDARD_OBJECT,
+      addFuncallableStandardClass(Symbol.FUNCALLABLE_STANDARD_OBJECT,
                        list(STANDARD_OBJECT, BuiltInClass.FUNCTION));
 
-  public static final StandardClass GENERIC_FUNCTION =
-    new FuncallableStandardClass(Symbol.GENERIC_FUNCTION,
-                                 list(METAOBJECT, FUNCALLABLE_STANDARD_OBJECT));
-  static {
-    addClass(Symbol.GENERIC_FUNCTION, GENERIC_FUNCTION);
-  }
-
   public static final StandardClass CLASS =
     addStandardClass(Symbol.CLASS, list(SPECIALIZER));
 
   public static final StandardClass BUILT_IN_CLASS =
     addStandardClass(Symbol.BUILT_IN_CLASS, list(CLASS));
 
-  public static final StandardClass FORWARD_REFERENCED_CLASS =
-    addStandardClass(Symbol.FORWARD_REFERENCED_CLASS, list(CLASS));
-
-  public static final StandardClass STRUCTURE_CLASS =
-    addStandardClass(Symbol.STRUCTURE_CLASS, list(CLASS));
-
   public static final StandardClass FUNCALLABLE_STANDARD_CLASS =
     addStandardClass(Symbol.FUNCALLABLE_STANDARD_CLASS, list(CLASS));
 
@@ -606,13 +569,6 @@ public class StandardClass extends SlotClass
   public static final StandardClass STANDARD_METHOD =
     addStandardClass(Symbol.STANDARD_METHOD, list(METHOD));
 
-  public static final StandardClass STANDARD_GENERIC_FUNCTION =
-    new StandardGenericFunctionClass();
-  static
-  {
-    addClass(Symbol.STANDARD_GENERIC_FUNCTION, STANDARD_GENERIC_FUNCTION);
-  }
-
   public static void initializeStandardClasses()
   {
     // We need to call setDirectSuperclass() here for classes that have a
@@ -621,16 +577,13 @@ public class StandardClass extends SlotClass
     STANDARD_CLASS.setDirectSuperclass(CLASS);
     STANDARD_OBJECT.setDirectSuperclass(BuiltInClass.CLASS_T);
     FUNCALLABLE_STANDARD_OBJECT.setDirectSuperclasses(list(STANDARD_OBJECT, BuiltInClass.FUNCTION));
-    GENERIC_FUNCTION.setDirectSuperclasses(list(METAOBJECT,
-                                                FUNCALLABLE_STANDARD_OBJECT));
-
     ARITHMETIC_ERROR.setCPL(ARITHMETIC_ERROR, ERROR, SERIOUS_CONDITION,
                             CONDITION, STANDARD_OBJECT, BuiltInClass.CLASS_T);
     ARITHMETIC_ERROR.setDirectSlotDefinitions(
       list(new SlotDefinition(Symbol.OPERATION,
-                               list(PACKAGE_CL.intern("ARITHMETIC-ERROR-OPERATION"))),
+                               list(Symbol.ARITHMETIC_ERROR_OPERATION)),
             new SlotDefinition(Symbol.OPERANDS,
-                               list(PACKAGE_CL.intern("ARITHMETIC-ERROR-OPERANDS")))));
+                               list(Symbol.ARITHMETIC_ERROR_OPERANDS))));
     BUILT_IN_CLASS.setCPL(BUILT_IN_CLASS, CLASS, SPECIALIZER, METAOBJECT, STANDARD_OBJECT,
                           BuiltInClass.CLASS_T);
     CELL_ERROR.setCPL(CELL_ERROR, ERROR, SERIOUS_CONDITION, CONDITION,
@@ -662,7 +615,7 @@ public class StandardClass extends SlotClass
                       STANDARD_OBJECT, BuiltInClass.CLASS_T);
     FILE_ERROR.setDirectSlotDefinitions(
       list(new SlotDefinition(Symbol.PATHNAME,
-                               list(PACKAGE_CL.intern("FILE-ERROR-PATHNAME")))));
+                               list(Symbol.FILE_ERROR_PATHNAME))));
     FLOATING_POINT_INEXACT.setCPL(FLOATING_POINT_INEXACT, ARITHMETIC_ERROR,
                                   ERROR, SERIOUS_CONDITION, CONDITION,
                                   STANDARD_OBJECT, BuiltInClass.CLASS_T);
@@ -676,20 +629,9 @@ public class StandardClass extends SlotClass
     FLOATING_POINT_UNDERFLOW.setCPL(FLOATING_POINT_UNDERFLOW, ARITHMETIC_ERROR,
                                     ERROR, SERIOUS_CONDITION, CONDITION,
                                     STANDARD_OBJECT, BuiltInClass.CLASS_T);
-    FORWARD_REFERENCED_CLASS.setCPL(FORWARD_REFERENCED_CLASS, CLASS,
-                                    SPECIALIZER, METAOBJECT, STANDARD_OBJECT, BuiltInClass.CLASS_T);
-    // Not all of these slots are necessary, but for now we take the
-    // standard layout.  Instances of this class will be redefined and
-    // get a new layout in due course.
-    FORWARD_REFERENCED_CLASS.setClassLayout(layoutStandardClass);
-    FORWARD_REFERENCED_CLASS.setDirectSlotDefinitions(standardClassSlotDefinitions());
     FUNCALLABLE_STANDARD_OBJECT.setCPL(FUNCALLABLE_STANDARD_OBJECT,
                                        STANDARD_OBJECT, BuiltInClass.FUNCTION,
                                        BuiltInClass.CLASS_T);
-    GENERIC_FUNCTION.setCPL(GENERIC_FUNCTION, METAOBJECT,
-                            FUNCALLABLE_STANDARD_OBJECT, STANDARD_OBJECT,
-                            BuiltInClass.FUNCTION,
-                            BuiltInClass.CLASS_T);
     JAVA_EXCEPTION.setCPL(JAVA_EXCEPTION, ERROR, SERIOUS_CONDITION, CONDITION,
                           STANDARD_OBJECT, BuiltInClass.CLASS_T);
     JAVA_EXCEPTION.setDirectSlotDefinitions(
@@ -716,14 +658,14 @@ public class StandardClass extends SlotClass
                          STANDARD_OBJECT, BuiltInClass.CLASS_T);
     PACKAGE_ERROR.setDirectSlotDefinitions(
       list(new SlotDefinition(Symbol.PACKAGE,
-                               list(PACKAGE_CL.intern("PACKAGE-ERROR-PACKAGE")))));
+                               list(Symbol.PACKAGE_ERROR_PACKAGE))));
     PARSE_ERROR.setCPL(PARSE_ERROR, ERROR, SERIOUS_CONDITION, CONDITION,
                        STANDARD_OBJECT, BuiltInClass.CLASS_T);
     PRINT_NOT_READABLE.setCPL(PRINT_NOT_READABLE, ERROR, SERIOUS_CONDITION,
                               CONDITION, STANDARD_OBJECT, BuiltInClass.CLASS_T);
     PRINT_NOT_READABLE.setDirectSlotDefinitions(
       list(new SlotDefinition(Symbol.OBJECT,
-                               list(PACKAGE_CL.intern("PRINT-NOT-READABLE-OBJECT")))));
+                               list(Symbol.PRINT_NOT_READABLE_OBJECT))));
     PROGRAM_ERROR.setCPL(PROGRAM_ERROR, ERROR, SERIOUS_CONDITION, CONDITION,
                          STANDARD_OBJECT, BuiltInClass.CLASS_T);
     READER_ERROR.setCPL(READER_ERROR, PARSE_ERROR, STREAM_ERROR, ERROR,
@@ -760,23 +702,21 @@ public class StandardClass extends SlotClass
                         STANDARD_OBJECT, BuiltInClass.CLASS_T);
     STREAM_ERROR.setDirectSlotDefinitions(
       list(new SlotDefinition(Symbol.STREAM,
-                               list(PACKAGE_CL.intern("STREAM-ERROR-STREAM")))));
-    STRUCTURE_CLASS.setCPL(STRUCTURE_CLASS, CLASS, SPECIALIZER, METAOBJECT,
-                           STANDARD_OBJECT, BuiltInClass.CLASS_T);
+                               list(Symbol.STREAM_ERROR_STREAM))));
     STYLE_WARNING.setCPL(STYLE_WARNING, WARNING, CONDITION, STANDARD_OBJECT,
                          BuiltInClass.CLASS_T);
     TYPE_ERROR.setCPL(TYPE_ERROR, ERROR, SERIOUS_CONDITION, CONDITION,
                       STANDARD_OBJECT, BuiltInClass.CLASS_T);
     TYPE_ERROR.setDirectSlotDefinitions(
       list(new SlotDefinition(Symbol.DATUM,
-                               list(PACKAGE_CL.intern("TYPE-ERROR-DATUM"))),
+                               list(Symbol.TYPE_ERROR_DATUM)),
             new SlotDefinition(Symbol.EXPECTED_TYPE,
-                               list(PACKAGE_CL.intern("TYPE-ERROR-EXPECTED-TYPE")))));
+                               list(Symbol.TYPE_ERROR_EXPECTED_TYPE))));
     UNBOUND_SLOT.setCPL(UNBOUND_SLOT, CELL_ERROR, ERROR, SERIOUS_CONDITION,
                         CONDITION, STANDARD_OBJECT, BuiltInClass.CLASS_T);
     UNBOUND_SLOT.setDirectSlotDefinitions(
       list(new SlotDefinition(Symbol.INSTANCE,
-                               list(PACKAGE_CL.intern("UNBOUND-SLOT-INSTANCE")))));
+                               list(Symbol.UNBOUND_SLOT_INSTANCE))));
     UNBOUND_VARIABLE.setCPL(UNBOUND_VARIABLE, CELL_ERROR, ERROR,
                             SERIOUS_CONDITION, CONDITION, STANDARD_OBJECT,
                             BuiltInClass.CLASS_T);
@@ -790,8 +730,6 @@ public class StandardClass extends SlotClass
     STANDARD_OBJECT.finalizeClass();
     FUNCALLABLE_STANDARD_OBJECT.finalizeClass();
     FUNCALLABLE_STANDARD_CLASS.finalizeClass();
-    FORWARD_REFERENCED_CLASS.finalizeClass();
-    GENERIC_FUNCTION.finalizeClass();
     ARITHMETIC_ERROR.finalizeClass();
     CELL_ERROR.finalizeClass();
     CONDITION.finalizeClass();
@@ -857,40 +795,5 @@ public class StandardClass extends SlotClass
                                               BuiltInClass.CLASS_T);
     STANDARD_EFFECTIVE_SLOT_DEFINITION.finalizeClass();
 
-    // STANDARD-GENERIC-FUNCTION
-    Debug.assertTrue(STANDARD_GENERIC_FUNCTION.isFinalized());
-    STANDARD_GENERIC_FUNCTION.setCPL(STANDARD_GENERIC_FUNCTION,
-                                     GENERIC_FUNCTION, METAOBJECT,
-                                     FUNCALLABLE_STANDARD_OBJECT,
-                                     STANDARD_OBJECT,
-                                     BuiltInClass.FUNCTION,
-                                     BuiltInClass.CLASS_T);
-    STANDARD_GENERIC_FUNCTION.setDirectSlotDefinitions(
-      list(new SlotDefinition(PACKAGE_SYS.intern("NAME"), NIL, constantlyNil),
-           new SlotDefinition(PACKAGE_SYS.intern("LAMBDA-LIST"), NIL,
-                              constantlyNil),
-           new SlotDefinition(PACKAGE_SYS.intern("REQUIRED-ARGS"), NIL,
-                              constantlyNil),
-           new SlotDefinition(PACKAGE_SYS.intern("OPTIONAL-ARGS"), NIL,
-                              constantlyNil),
-           new SlotDefinition(PACKAGE_SYS.intern("INITIAL-METHODS"), NIL,
-                              constantlyNil),
-           new SlotDefinition(PACKAGE_SYS.intern("METHODS"), NIL,
-                              constantlyNil),
-           new SlotDefinition(PACKAGE_SYS.intern("METHOD-CLASS"), NIL,
-                              constantlyNil),
-           new SlotDefinition(PACKAGE_SYS.intern("%METHOD-COMBINATION"), NIL,
-                              constantlyNil,
-                              list(internKeyword("METHOD-COMBINATION"))),
-           new SlotDefinition(PACKAGE_SYS.intern("ARGUMENT-PRECEDENCE-ORDER"),
-                              NIL, constantlyNil),
-           new SlotDefinition(PACKAGE_SYS.intern("DECLARATIONS"), NIL,
-                              constantlyNil),
-           new SlotDefinition(PACKAGE_SYS.intern("CLASSES-TO-EMF-TABLE"), NIL,
-                              constantlyNil),
-           new SlotDefinition(Symbol._DOCUMENTATION, NIL, constantlyNil,
-                              list(internKeyword("DOCUMENTATION")))));
-    // There are no inherited slots.
-    STANDARD_GENERIC_FUNCTION.setSlotDefinitions(STANDARD_GENERIC_FUNCTION.getDirectSlotDefinitions());
   }
 }
