@@ -2,7 +2,7 @@
  * StandardObject.java
  *
  * Copyright (C) 2003-2006 Peter Graves
- * $Id: StandardObject.java 14137 2012-08-26 19:23:15Z rschlatte $
+ * $Id: StandardObject.java 14499 2013-05-15 06:45:13Z rschlatte $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -211,9 +211,9 @@ public class StandardObject extends LispObject
     return unreadableString(typeOf().printObject());
   }
 
-  Layout updateLayout()
+  synchronized Layout updateLayout()
   {
-    Debug.assertTrue(layout.isInvalid());
+    if (!layout.isInvalid()) return layout;
     Layout oldLayout = layout;
     LispObject cls = oldLayout.getLispClass();
     Layout newLayout;
@@ -302,9 +302,18 @@ public class StandardObject extends LispObject
       }
     Debug.assertTrue(layout != null);
     int index = layout.getSlotIndex(slotName);
-    //### FIXME: should call SLOT-MISSING (clhs)
-    if (index < 0)
-      return error(new LispError("Missing slot " + slotName.princToString()));
+    if (index < 0) {
+      // Not found.
+      final LispThread thread = LispThread.currentThread();
+      // If the operation is slot-value, only the primary value [of
+      // slot-missing] will be used by the caller, and all other values
+      // will be ignored.
+      LispObject value = thread.execute(Symbol.SLOT_MISSING,
+                                        this.getLispClass(), this,
+                                        slotName, Symbol.SLOT_VALUE);
+      thread._values = null;
+      return value;
+    }
     return slots[index];
   }
 
@@ -320,9 +329,15 @@ public class StandardObject extends LispObject
       }
     Debug.assertTrue(layout != null);
     int index = layout.getSlotIndex(slotName);
-    // FIXME: should call SLOT-MISSING (clhs)
-    if (index < 0)
-      error(new LispError("Missing slot " + slotName.princToString()));
+    if (index < 0) {
+      // Not found.
+      final LispThread thread = LispThread.currentThread();
+      // If the operation is setf or slot-makunbound, any values
+      // [returned by slot-missing] will be ignored by the caller.
+      thread.execute(Symbol.SLOT_MISSING, this.getLispClass(), this,
+                     slotName, Symbol.SETF, newValue);
+      thread._values = null;
+    }
     slots[index] = newValue;
   }
   
@@ -448,9 +463,8 @@ public class StandardObject extends LispObject
                             list(Symbol.INTEGER, Fixnum.ZERO,
                                  Fixnum.getInstance(instance.slots.length - 1)));
         else
-          return error(new ProgramError("The object "
-                                        + instance.princToString() +
-                                        " has no slots."));
+          return program_error("The object " + instance.princToString()
+                               + " has no slots.");
 
       }
       // We let UNBOUND_VALUE escape here, since invoking
@@ -495,10 +509,8 @@ public class StandardObject extends LispObject
                             list(Symbol.INTEGER, Fixnum.ZERO,
                                  Fixnum.getInstance(instance.slots.length - 1)));
         else
-          return error(new ProgramError("The object "
-                                        + instance.princToString() +
-                                        " has no slots."));
-
+          return program_error("The object " + instance.princToString()
+                               + " has no slots.");
       }
       return third;
     }
@@ -656,18 +668,27 @@ public class StandardObject extends LispObject
     @Override
     public LispObject execute(LispObject arg)
     {
-      if (arg == StandardClass.STANDARD_CLASS)
+      if (arg == StandardClass.FUNCALLABLE_STANDARD_CLASS) {
+        return new FuncallableStandardClass();
+      } else if (arg == StandardClass.STANDARD_CLASS) {
         return new StandardClass();
-      if (arg instanceof StandardClass)
-        return ((StandardClass)arg).allocateInstance();
-      if (arg.typep(StandardClass.STANDARD_CLASS) != NIL) {
+      } else if (arg instanceof StandardClass) {
+        StandardClass cls = (StandardClass)arg;
+        Layout layout = cls.getClassLayout();
+        if (layout == null) {
+          program_error("No layout for class " + cls.princToString() + ".");
+        }
+        return new StandardObject(cls, layout.getLength());
+      } else if (arg.typep(StandardClass.STANDARD_CLASS) != NIL) {
         LispObject l = Symbol.CLASS_LAYOUT.execute(arg);
-        if (! (l instanceof Layout))
-          return error(new ProgramError("Invalid standard class layout for: " + arg.princToString()));
-        
+        if (! (l instanceof Layout)) {
+          program_error("Invalid standard class layout for class "
+                        + arg.princToString() + ".");
+        }
         return new StandardObject((Layout)l);
+      } else {
+        return type_error(arg, Symbol.STANDARD_CLASS);
       }
-      return type_error(arg, Symbol.STANDARD_CLASS);
     }
   };
 }
