@@ -2,7 +2,7 @@
  * Stream.java
  *
  * Copyright (C) 2003-2007 Peter Graves
- * $Id: Stream.java 14358 2013-01-16 12:28:40Z mevenson $
+ * $Id: Stream.java 14466 2013-04-24 12:50:40Z rschlatte $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -336,6 +336,42 @@ public class Stream extends StructureObject {
             ((DecodingReader)reader).setCharset(Charset.forName(encoding));
     }
 
+  public static final Primitive STREAM_EXTERNAL_FORMAT = new pf_stream_external_format();
+  @DocString(
+    name="stream-external-format", 
+    args="stream",
+    doc="Returns the external format of STREAM."
+  )
+  private static final class pf_stream_external_format extends Primitive {
+    pf_stream_external_format() {
+      super("stream-external-format", "stream");
+    }
+    public LispObject execute(LispObject arg) {
+      if (arg instanceof Stream) {
+        return ((Stream)arg).getExternalFormat();
+      } else {
+        return type_error(arg, Symbol.STREAM);
+      }
+    }
+  }
+
+  // DEFSETF-ed in 'setf.lisp'
+  public static final Primitive SET_STREAM_EXTERNAL_FORMAT = new pf__set_stream_external_format();
+  @DocString(
+    name="%set-stream-external-format",
+    args="stream format"
+  )
+  private static final class pf__set_stream_external_format extends Primitive {
+    pf__set_stream_external_format() {
+        super("%set-stream-external-format",
+              PACKAGE_SYS, false, "stream external-format");
+    }
+    public LispObject execute(LispObject stream, LispObject format) {
+      Stream s = checkStream(stream);
+      s.setExternalFormat(format);
+      return format;
+    }
+  };
 
   public static final Primitive AVAILABLE_ENCODINGS = new pf_available_encodings();
   @DocString(name="available-encodings",
@@ -1139,7 +1175,7 @@ public class Stream extends StructureObject {
                 if (invert)
                     packageName = invert(packageName, packageFlags);
 
-                pkg = Packages.findPackage(packageName);
+                pkg = getCurrentPackage().findPackage(packageName);
                 if (pkg == null)
                     return error(new ReaderError("The package \"" + packageName + "\" can't be found.", this));
             }
@@ -2197,8 +2233,8 @@ public class Stream extends StructureObject {
             final Stream stream = checkStream(first);
             if (second == Keyword.ABORT)
                 return stream.close(third);
-            return error(new ProgramError("Unrecognized keyword argument " +
-                                          second.princToString() + "."));
+            return program_error("Unrecognized keyword argument "
+                                 + second.princToString() + ".");
         }
     };
 
