@@ -1,7 +1,7 @@
 ;;; compiler-pass1.lisp
 ;;;
 ;;; Copyright (C) 2003-2008 Peter Graves
-;;; $Id: compiler-pass1.lisp 14147 2012-09-02 11:38:30Z ehuelsmann $
+;;; $Id: compiler-pass1.lisp 14558 2013-06-22 19:57:58Z ehuelsmann $
 ;;;
 ;;; This program is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU General Public License
@@ -41,6 +41,7 @@
 (require "DUMP-FORM")
 (require "JAVA")
 
+(proclaim '(optimize speed))
 
 (defun generate-inline-expansion (name lambda-list body
                                   &optional (args nil args-p))
@@ -444,68 +445,61 @@ where each of the vars returned is a list with these elements:
 (declaim (ftype (function (t) t) p1-body))
 (defun p1-body (body)
   (declare (optimize speed))
-  (let ((tail body))
-    (loop
-      (when (endp tail)
-        (return))
-      (setf (car tail) (p1 (%car tail)))
-      (setf tail (%cdr tail))))
-  body)
+  (loop
+     for form in body
+     for processed-form = (p1 form)
+     collect processed-form
+     while (not (and (consp processed-form)
+                     (memq (car processed-form) '(GO RETURN-FROM THROW))))))
 
 (defknown p1-default (t) t)
 (declaim (inline p1-default))
 (defun p1-default (form)
-  (setf (cdr form) (p1-body (cdr form)))
-  form)
+  (declare (optimize speed))
+  (cons (car form) (loop for f in (cdr form) collect (p1 f))))
 
-(defmacro p1-let/let*-vars
-    (block varlist variables-var var body1 body2)
-  (let ((varspec (gensym))
-        (initform (gensym))
-        (name (gensym)))
-    `(let ((,variables-var ()))
-       (dolist (,varspec ,varlist)
-         (cond ((consp ,varspec)
-                ;; Even though the precompiler already signals this
-                ;; error, double checking can't hurt; after all, we're
-                ;; also rewriting &AUX into LET* bindings.
-                (unless (<= 1 (length ,varspec) 2)
-                  (compiler-error "The LET/LET* binding specification ~S is invalid."
-                                  ,varspec))
-                (let* ((,name (%car ,varspec))
-                       (,initform (p1 (%cadr ,varspec)))
-                       (,var (make-variable :name (check-name ,name)
-                                            :initform ,initform
-                                            :block ,block)))
-                  (when (neq ,initform (cadr ,varspec))
-                    (setf (cadr ,varspec) ,initform))
-                  (push ,var ,variables-var)
-                  ,@body1))
-               (t
-                (let ((,var (make-variable :name (check-name ,varspec)
-                                           :block ,block)))
-                  (push ,var ,variables-var)
-                  ,@body1))))
-       ,@body2)))
+(defun let/let*-variables (block bindings)
+  (loop for binding in bindings
+     if (consp binding)
+     collect (make-variable :name (check-name (car binding))
+                            :initform (cadr binding)
+                            :block block)
+     else
+     collect (make-variable :name (check-name binding)
+                            :block block)))
+
+(defun valid-let/let*-binding-p (varspec)
+  (when (consp varspec)
+    (unless (<= 1 (length varspec) 2)
+      (compiler-error "The LET/LET* binding specification ~
+                       ~S is invalid." varspec)))
+  T)
+
+(defun check-let/let*-bindings (bindings)
+  (every #'valid-let/let*-binding-p bindings))
 
 (defknown p1-let-vars (t) t)
 (defun p1-let-vars (block varlist)
-  (p1-let/let*-vars block
-   varlist vars var
-   ()
-   ((setf vars (nreverse vars))
+  (check-let/let*-bindings varlist)
+  (let ((vars (let/let*-variables block varlist)))
+    (dolist (variable vars)
+      (setf (variable-initform variable)
+            (p1 (variable-initform variable))))
     (dolist (variable vars)
       (push variable *visible-variables*)
       (push variable *all-variables*))
-    vars)))
+    vars))
 
 (defknown p1-let*-vars (t) t)
 (defun p1-let*-vars (block varlist)
-  (p1-let/let*-vars block
-   varlist vars var
-   ((push var *visible-variables*)
-    (push var *all-variables*))
-   ((nreverse vars))))
+  (check-let/let*-bindings varlist)
+  (let ((vars (let/let*-variables block varlist)))
+    (dolist (variable vars)
+      (setf (variable-initform variable)
+            (p1 (variable-initform variable)))
+      (push variable *visible-variables*)
+      (push variable *all-variables*))
+    vars))
 
 (defun p1-let/let* (form)
   (declare (type cons form))
@@ -605,8 +599,8 @@ where each of the vars returned is a list with these elements:
 (defun p1-block (form)
   (let* ((block (make-block-node (cadr form)))
          (*block* block)
-         (*blocks* (cons block *blocks*)))
-    (setf (cddr form) (p1-body (cddr form)))
+         (*blocks* (cons block *blocks*))
+         (form (list* (car form) (cadr form) (p1-body (cddr form)))))
     (setf (block-form block) form)
     (when (block-non-local-return-p block)
       ;; Add a closure variable for RETURN-FROM to use
@@ -646,16 +640,25 @@ where each of the vars returned is a list with these elements:
          (body (cddr form))
          (block (make-synchronized-node))
          (*block* block)
-         (*blocks* (cons block *blocks*))
-         result)
-    (dolist (subform body)
-      (let ((op (and (consp subform) (%car subform))))
-        (push (p1 subform) result)
-        (when (memq op '(GO RETURN-FROM THROW))
-          (return))))
+         (*blocks* (cons block *blocks*)))
     (setf (synchronized-form block)
           (list* 'threads:synchronized-on synchronized-object
-                 (nreverse result)))
+                 (p1-body body)))
+    block))
+
+(defun p1-java-jrun-exception-protected (form)
+  (assert (eq (first form) 'java:jrun-exception-protected))
+  (assert (or (eq (car (second form)) 'lambda)
+              (and (eq (car (second form)) 'function)
+                   (eq (car (second (second form))) 'lambda))))
+  (let* ((body (if (eq (car (second form)) 'lambda)
+                   (cddr (second form))
+                   (cddr (second (second form)))))
+         (block (make-exception-protected-node))
+         (*block* block)
+         (*blocks* (cons block *blocks*)))
+    (setf (exception-protected-form block)
+          (p1-body body))
     block))
 
 (defun p1-unwind-protect (form)
@@ -1379,6 +1382,8 @@ where each of the vars returned is a list with these elements:
                   (UNWIND-PROTECT       p1-unwind-protect)
                   (THREADS:SYNCHRONIZED-ON
                                         p1-threads-synchronized-on)
+                  (JAVA:JRUN-EXCEPTION-PROTECTED
+                                        p1-java-jrun-exception-protected)
                   (JVM::WITH-INLINE-CODE identity)))
     (install-p1-handler (%car pair) (%cadr pair))))
 
