@@ -1,7 +1,7 @@
 ;;; gray-streams.lisp
 ;;;
 ;;; Copyright (C) 2004-2007 Peter Graves, Andras Simon
-;;; $Id: gray-streams.lisp 14076 2012-08-13 06:22:13Z ehuelsmann $
+;;; $Id: gray-streams.lisp 14605 2014-01-19 19:13:56Z mevenson $
 ;;;
 ;;; This program is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU General Public License
@@ -114,6 +114,8 @@
 ;;;; Much of the implementation of the Gray streams below is from the
 ;;;; document referenced earlier.
 ;;;;
+(require "PPRINT")
+
 (defpackage "GRAY-STREAMS"
   (:use
    "COMMON-LISP")
@@ -312,6 +314,10 @@
   (declare (ignore stream))
   nil)
 
+(defmethod stream-finish-output (stream)
+  (declare (ignore stream))
+  nil)
+
 (defmethod stream-clear-output (stream)
   (declare (ignore stream))
   nil)
@@ -341,45 +347,43 @@
       (dotimes (i (- current column) t)
         (stream-write-char stream #\Space)))))
 
+(defun basic-read-sequence (stream sequence start end
+                            expected-element-type read-fun)
+  (let ((element-type (stream-element-type stream)))
+    (if (subtypep element-type expected-element-type)
+        (dotimes (count (- end start)
+                  ;; If (< end start), skip the dotimes body but
+                  ;; return start
+                  (max start end))
+          (let ((el (funcall read-fun stream)))
+            (when (eq el :eof)
+              (return (+ count start)))
+            (setf (elt sequence (+ count start)) el)))
+        (error "Cannot READ-SEQUENCE on stream of :ELEMENT-TYPE ~A"
+               element-type))))
+
+(defun basic-write-sequence (stream sequence start end
+                             expected-element-type write-fun)
+  (let ((element-type (stream-element-type stream)))
+    (if (subtypep element-type expected-element-type)
+        ;; Avoid LOOP because it isn't loaded yet
+        (do ((n start (+ n 1)))
+            ((= n end))
+          (funcall write-fun stream (elt sequence n)))
+        (error "Cannot WRITE-SEQUENCE on stream of :ELEMENT-TYPE ~A"
+               element-type)))
+  (stream-force-output stream)
+  sequence)
+
 (defmethod stream-read-sequence ((stream  fundamental-character-input-stream)
                                  sequence &optional (start 0) end)
-  (let ((element-type (stream-element-type stream))
-        (end (or end (length sequence)))
-        (eof (cons nil nil)))
-    (cond
-     ((eq element-type 'character)
-      (dotimes (count (- end start) (- end start))
-        (let ((c (stream-read-char stream nil eof)))
-          (if (eq c eof)
-              (return (+ count start)))
-          (setf (elt sequence (+ count start)) c))))
-     ((or (eq element-type 'byte)
-          (eq element-type 'unsigned-byte)
-          (eq element-type 'signed-byte))
-      (dotimes (count (- end start) (- end start))
-        (let ((b (stream-read-byte stream nil eof)))
-          (if (eq b eof)
-              (return (+ count start)))
-          (setf (elt sequence (+ count start)) b))))
-     (t (error "Cannot READ-SEQUENCE on stream of :ELEMENT-TYPE ~A"
-               element-type)))))
+  (basic-read-sequence stream sequence start (or end (length sequence))
+                       'character #'stream-read-char))
 
 (defmethod stream-write-sequence ((stream fundamental-character-output-stream)
                                   sequence &optional (start 0) end)
-  (let ((element-type (stream-element-type stream))
-        (end (or end (length sequence))))
-    (if (eq element-type 'character)
-        (do ((n start (+ n 1)))
-            ((= n end))
-          (stream-write-char
-           stream
-           (if (typep (elt sequence n) 'number)
-               (#+nil ccl:int-char code-char (elt sequence n))
-               (elt sequence n))))
-        (do ((n start (+ n 1)))
-            ((= n end))
-          (stream-write-byte (elt sequence n) stream))))    ;; recoded to avoid LOOP, because it isn't loaded yet
-  (stream-force-output stream))
+  (basic-write-sequence stream sequence start (or end (length sequence))
+                        'character #'stream-write-char))
 
 (defclass fundamental-binary-input-stream
   (fundamental-input-stream fundamental-binary-stream))
@@ -387,6 +391,16 @@
 (defclass fundamental-binary-output-stream
   (fundamental-output-stream fundamental-binary-stream))
 
+(defmethod stream-read-sequence ((stream fundamental-binary-input-stream)
+                                 sequence &optional (start 0) end)
+  (basic-read-sequence stream sequence start (or end (length sequence))
+                       'signed-byte #'stream-read-byte))
+
+(defmethod stream-write-sequence ((stream fundamental-binary-output-stream)
+                                  sequence &optional (start 0) end)
+  (basic-write-sequence stream sequence start (or end (length sequence))
+                        'signed-byte #'stream-write-byte))
+
 (defun decode-read-arg (arg)
   (cond ((null arg) *standard-input*)
         ((eq arg t) *terminal-io*)
@@ -646,4 +660,31 @@
 (setf (symbol-function 'common-lisp::two-way-stream-output-stream) #'gray-two-way-stream-output-stream)
 |#
 
+(eval-when (:load-toplevel)
+  (mapcar (lambda (o) (mop:finalize-inheritance (find-class o)))
+          '(fundamental-stream 
+            fundamental-input-stream fundamental-output-stream
+            fundamental-character-stream 
+            fundamental-character-input-stream fundamental-character-output-stream
+            fundamental-binary-stream
+            fundamental-binary-input-stream fundamental-binary-output-stream)))
+ 
 (provide 'gray-streams)
+
+;;; Fixup Gray/ANSI stream relations 
+
+(defparameter *sys--stream-charpos* #'sys::stream-charpos)
+(defun sys::stream-charpos (stream)
+  (cond         
+    ((subtypep (type-of stream) 'gray-streams:fundamental-stream)
+     (stream-line-column stream))
+    ((streamp stream)
+     (funcall *sys--stream-charpos* stream))))
+
+(defparameter *sys--stream-%set-charpos* #'sys::stream-%set-charpos)
+(defun sys::stream-%set-charpos (new-value stream)
+  (cond 
+    ((subtypep (type-of stream) 'gray-streams:fundamental-stream)
+     (setf (stream-line-column stream) new-value))
+    ((streamp stream)
+     (funcall *sys--stream-%set-charpos* stream new-value))))
