@@ -1,7 +1,7 @@
 ;;; jvm-class-file.lisp
 ;;;
 ;;; Copyright (C) 2010 Erik Huelsmann
-;;; $Id: jvm-class-file.lisp 14096 2012-08-15 22:55:27Z ehuelsmann $
+;;; $Id: jvm-class-file.lisp 14552 2013-06-18 19:20:51Z ehuelsmann $
 ;;;
 ;;; This program is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU General Public License
@@ -158,6 +158,8 @@ initialized from the `java-dotted-name'."
 (define-class-name +java-object+ "java.lang.Object")
 (define-class-name +java-string+ "java.lang.String")
 (define-class-name +java-system+ "java.lang.System")
+(define-class-name +java-stack-overflow+ "java.lang.StackOverflowError")
+(define-class-name +java-out-of-memory+ "java.lang.OutOfMemoryError")
 (define-class-name +java-io-input-stream+ "java.io.InputStream")
 (define-class-name +java-util-collection+ "java.util.Collection")
 (define-class-name +lisp-object+ "org.armedbear.lisp.LispObject")
