@@ -1,7 +1,7 @@
 ;;; jvm.lisp
 ;;;
 ;;; Copyright (C) 2003-2008 Peter Graves
-;;; $Id: jvm.lisp 14121 2012-08-18 11:37:34Z ehuelsmann $
+;;; $Id: jvm.lisp 14552 2013-06-18 19:20:51Z ehuelsmann $
 ;;;
 ;;; This program is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU General Public License
@@ -40,7 +40,10 @@
 (defvar *enable-dformat* nil)
 (defvar *callbacks* nil
   "A list of functions to be called by the compiler and code generator
-in order to generate 'compilation events'.")
+in order to generate 'compilation events'.
+
+A callback function takes five arguments:  
+CALLBACK-TYPE CLASS PARENT CONTENT EXCEPTION-HANDLERS.")
 
 (declaim (inline invoke-callbacks))
 (defun invoke-callbacks (&rest args)
@@ -266,6 +269,15 @@ of the compilands being processed (p1: so far; p2: in total).")
   (compiland *current-compiland*)
   block)
 
+
+(defmethod print-object ((object jvm::variable-info) stream)
+  (print-unreadable-object (object stream :type t :identity t)
+    (princ (jvm::variable-name object) stream)
+    (princ " in " stream)
+    (princ (jvm::compiland-name (jvm::variable-compiland object)) stream)))
+
+
+
 (defstruct (var-ref (:constructor make-var-ref (variable)))
   ;; The variable this reference refers to. Will be NIL if the VAR-REF has been
   ;; rewritten to reference a constant value.
@@ -275,6 +287,11 @@ of the compilands being processed (p1: so far; p2: in total).")
   ;; The constant value of this VAR-REF.
   constant-value)
 
+(defmethod print-object ((object jvm::var-ref) stream)
+  (print-unreadable-object (object stream :type t :identity t)
+    (princ "ref ")
+    (print-object (jvm::var-ref-variable object) stream)))
+
 ;; obj can be a symbol or variable
 ;; returns variable or nil
 (declaim (ftype (function (t) t) unboxed-fixnum-variable))
@@ -556,6 +573,19 @@ if that parent belongs to the same compiland."
     (add-node-child *block* block)
     block))
 
+
+(defstruct (exception-protected-node
+             (:conc-name exception-protected-)
+             (:include protected-node)
+             (:constructor %make-exception-protected-node ())))
+(defknown make-exception-protected-node () t)
+(defun make-exception-protected-node ()
+  (let ((block (%make-exception-protected-node)))
+    (push block (compiland-blocks *current-compiland*))
+    (add-node-child *block* block)
+    block))
+
+
 (defun find-block (name)
   (dolist (block *blocks*)
     (when (and (block-node-p block)
