@@ -1,7 +1,7 @@
 ;;; late-setf.lisp
 ;;;
 ;;; Copyright (C) 2003-2005 Peter Graves
-;;; $Id$
+;;; $Id: late-setf.lisp 14727 2014-09-09 10:54:55Z mevenson $
 ;;;
 ;;; This program is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU General Public License
@@ -41,7 +41,7 @@
 			 (parse-defmacro lambda-list whole body access-fn
 					 'define-setf-expander
 					 :environment environment)
-      `(progn
+      `(eval-when (:compile-toplevel :load-toplevel :execute)
          ,@(when doc
              `((%set-documentation ',access-fn 'setf ,doc)))
          (setf (get ',access-fn 'setf-expander)
