@@ -1,7 +1,7 @@
 ;;; precompiler.lisp
 ;;;
 ;;; Copyright (C) 2003-2008 Peter Graves <peter@armedbear.org>
-;;; $Id$
+;;; $Id: precompiler.lisp 14763 2015-04-14 15:42:27Z ehuelsmann $
 ;;;
 ;;; This program is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU General Public License
@@ -335,13 +335,16 @@
 ;; Special precompilation in precompile-when and precompile-unless
 ;; No precompilation in precompile-nth-value
 ;; Special precompilation in precompile-return
-;; Special precompilation in expand-macro
 ;;
 ;; if *in-jvm-compile* is false
 
 (defvar *in-jvm-compile* nil)
 (defvar *precompile-env* nil)
 
+(declaim (inline expand-macro))
+(defun expand-macro (form)
+  (macroexpand-1 form *precompile-env*))
+
 
 (declaim (ftype (function (t) t) precompile1))
 (defun precompile1 (form)
@@ -795,24 +798,37 @@
              (find-use name (%cdr expression))))))
 
 (defun precompile-flet/labels (form)
-  (let ((*precompile-env* (make-environment *precompile-env*))
-        (operator (car form))
-        (locals (cadr form))
-	body)
-    ;; first augment the environment with the newly-defined local functions
+  (let* ((*precompile-env* (make-environment *precompile-env*))
+         (operator (car form))
+         (locals (cadr form))
+         precompiled-locals
+         applicable-locals
+         body)
+    (when (eq operator 'FLET)
+       ;; FLET functions *don't* shadow within their own FLET form
+       (setf precompiled-locals
+             (precompile-local-functions locals))
+       (setf applicable-locals precompiled-locals))
+    ;; augment the environment with the newly-defined local functions
     ;; to shadow preexisting macro definitions with the same names
     (dolist (local locals)
+      ;; we can use the non-precompiled locals, because the function body isn't used
       (environment-add-function-definition *precompile-env*
 					   (car local) (cddr local)))
+    (when (eq operator 'LABELS)
+       ;; LABELS functions *do* shadow within their own LABELS form
+       (setf precompiled-locals
+             (precompile-local-functions locals))
+       (setf applicable-locals precompiled-locals))
     ;; then precompile (thus macro-expand) the body before inspecting it
-    ;; for the use of our locals and optimizing them away
+    ;; for the use of our locals and eliminating dead code
     (setq body (mapcar #'precompile1 (cddr form)))
-    (dolist (local locals)
+    (dolist (local precompiled-locals)
       (let* ((name (car local))
              (used-p (find-use name body)))
         (unless used-p
           (when (eq operator 'LABELS)
-            (dolist (local locals)
+            (dolist (local precompiled-locals)
               (when (neq name (car local))
                 (when (find-use name (cddr local))
                   (setf used-p t)
@@ -824,21 +840,17 @@
                              (cdr (memq '&optional (cadr local)))
                              (cdr (memq '&key (cadr local)))
                              (cdr (memq '&aux (cadr local))))))
-                  (when (and vars (find-use name vars)
-                             (setf used-p t)
-                             (return))))))))
+                  (when (and vars (find-use name vars))
+                    (setf used-p t)
+                    (return)))
+                ))))
         (unless used-p
           (format t "; Note: deleting unused local function ~A ~S~%"
                   operator name)
-          (let* ((new-locals (remove local locals :test 'eq))
-                 (new-form
-                  (if new-locals
-                      (list* operator new-locals body)
-                      (list* 'LOCALLY body))))
-            (return-from precompile-flet/labels (precompile1 new-form))))))
-    (list* (car form)
-           (precompile-local-functions locals)
-           body)))
+          (setf applicable-locals (remove local applicable-locals)))))
+    (if applicable-locals
+        (list* operator applicable-locals body)
+        (list* 'LOCALLY body))))
 
 (defun precompile-function (form)
   (if (and (consp (cadr form)) (eq (caadr form) 'LAMBDA))
@@ -943,24 +955,6 @@
          (precompile1 (cadr form))
          (mapcar #'precompile1 (cddr form))))
 
-;; EXPAND-MACRO is like MACROEXPAND, but EXPAND-MACRO quits if *IN-JVM-COMPILE*
-;; is false and a macro is encountered that is also implemented as a special
-;; operator, so interpreted code can use the special operator implementation.
-(defun expand-macro (form)
-  (let (exp)
-    (loop
-       (unless *in-jvm-compile*
-         (when (and (consp form)
-                    (symbolp (%car form))
-                    (special-operator-p (%car form)))
-           (return-from expand-macro (values form exp))))
-       (multiple-value-bind (result expanded)
-           (macroexpand-1 form *precompile-env*)
-         (unless expanded
-           (return-from expand-macro (values result exp)))
-         (setf form result
-               exp t)))))
-
 (declaim (ftype (function (t t) t) precompile-form))
 (defun precompile-form (form in-jvm-compile
                         &optional precompile-env)
