@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2008 Hideo at Yokohama
  * Copyright (C) 2008-2009 Erik Huelsmann
- * $Id: RandomAccessCharacterFile.java 13088 2010-12-05 07:50:47Z mevenson $
+ * $Id: RandomAccessCharacterFile.java 14735 2014-10-29 00:13:21Z mevenson $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -50,7 +50,11 @@ import java.nio.charset.CharsetDecoder;
 import java.nio.charset.CharsetEncoder;
 import java.nio.charset.CoderResult;
 import java.nio.charset.CodingErrorAction;
+import java.nio.charset.UnsupportedCharsetException;
 import org.armedbear.lisp.Debug;
+import static org.armedbear.lisp.Lisp.error;
+import org.armedbear.lisp.SimpleError;
+import org.armedbear.lisp.SimpleString;
 
 public class RandomAccessCharacterFile {
 
@@ -305,12 +309,19 @@ public class RandomAccessCharacterFile {
     }
 
     public void setEncoding(String encoding) {
-        cset = (encoding == null)
-            ? Charset.defaultCharset() : Charset.forName(encoding);
-        cdec = cset.newDecoder();
-        cdec.onMalformedInput(CodingErrorAction.REPLACE);
-        cdec.onUnmappableCharacter(CodingErrorAction.REPLACE);
-        cenc = cset.newEncoder();
+      if (encoding == null) {
+        cset = Charset.defaultCharset();
+      } else {
+        try {
+          cset = Charset.forName(encoding);
+        } catch (UnsupportedCharsetException e) {
+          error(new SimpleError("Undefined encoding: " + encoding));
+        }
+      }
+      cdec = cset.newDecoder();
+      cdec.onMalformedInput(CodingErrorAction.REPLACE);
+      cdec.onUnmappableCharacter(CodingErrorAction.REPLACE);
+      cenc = cset.newEncoder();
     }
 
     public Writer getWriter() {
